"""
There are two kinds of Repositories in PyChemia

Structure Repositories where many structures are stored
Execution Repositories where  the out of every calculation
is stored

Each structure contains some metadata that is accessible with
the StructureEntry object
Also each calculation has it own metadata accessible by ExecutionEntry
object
"""
import hashlib
import json as _json
import os
import uuid as _uuid
import shutil as _shutil
import math
from pychemia.core.structure import load_structure_json
from pychemia.utils.computing import deep_unicode


class StructureEntry:
    """
    Defines one entry in the repository of Structures
    """

    def __init__(self, structure=None, repository=None, identifier=None, original_file=None, tags=None):
        """
        Creates a new Entry for Structures
        If identifier is provided the corresponding Structure is load in the Entry
        Otherwise a new entry is created with a UUID random identifier

        Args:
        identifier: (string) UUID identifier for a structure
        repository: (object) The StructureRepository that will be associated
        original_file: (string) Path to the original file (CIF, POSCAR, etc)
        tags: (string or list) Tags that will be associated to that structure
        """
        self.properties = None

        if identifier is None:
            self.structure = structure
            self.identifier = str(_uuid.uuid4())
            self.path = None
            if original_file is not None:
                assert (os.path.isfile(original_file))
            self.original_file = original_file
            self.parents = []
            self.children = []
            if isinstance(tags, str):
                self.tags = [tags]
            elif isinstance(tags, list):
                self.tags = tags
            elif tags is None:
                self.tags = []
            else:
                raise ValueError('The variable tags must be a string or list of strings')

            if len(self.structure.composition) == 1:
                self.add_tags('pure')
            elif len(self.structure.composition) == 2:
                self.add_tags('binary')
            elif len(self.structure.composition) == 3:
                self.add_tags('ternary')
            elif len(self.structure.composition) == 4:
                self.add_tags('quaternary')

        else:
            assert (original_file is None)
            assert (structure is None)
            assert (tags is None)
            assert (repository is not None)
            self.identifier = identifier
            self.repository = repository
            self.path = self.repository.path + '/' + self.identifier
            if not os.path.isdir(self.path):
                raise ValueError("Directory not found: " + self.path)
            if not os.path.isfile(self.path + '/metadata.json'):
                raise ValueError("No metadata found in " + self.path)
            if not os.path.isfile(self.path + '/structure.json'):
                raise ValueError("No structure found in " + self.path)
            self.load()

    def metadatatodict(self):
        ret = {'tags': self.tags,
               'parents': self.parents,
               'children': self.children}
        return ret

    def load(self):
        assert isinstance(self.identifier, str)
        rf = open(self.path + '/metadata.json', 'r')
        self.metadatafromdict(deep_unicode(_json.load(rf)))
        rf.close()
        if self.tags is None:
            self.tags = []
        if self.children is None:
            self.children = []
        if self.parents is None:
            self.parents = []
        self.structure = load_structure_json(self.path + '/structure.json')
        if os.path.isfile(self.path + '/properties.json'):
            rf = open(self.path + '/properties.json', 'r')
            try:
                self.properties = deep_unicode(_json.load(rf))
            except ValueError:
                os.rename(self.path + '/properties.json', self.path + '/properties.json.FAILED')
                self.properties = None
            rf.close()
        self.load_originals()

    def load_originals(self):
        orig_dir = self.path + '/original'
        if os.path.isdir(orig_dir):
            self.original_file = [os.path.abspath(orig_dir + '/' + x) for x in os.listdir(orig_dir)]
        else:
            self.original_file = []

    def save(self):
        if self.path is None:
            self.path = self.repository.path + '/' + self.identifier
        wf = open(self.path + '/metadata.json', 'w')
        _json.dump(self.metadatatodict(), wf, sort_keys=True, indent=4, separators=(',', ': '))
        wf.close()
        self.structure.save_json(self.path + '/structure.json')
        if self.properties is not None:
            wf = open(self.path + '/properties.json', 'w')
            _json.dump(self.properties, wf, sort_keys=True, indent=4, separators=(',', ': '))
            wf.close()
        if self.original_file is not None:
            self.add_original_file(self.original_file)

    def metadatafromdict(self, entrydict):
        self.tags = entrydict['tags']
        self.parents = entrydict['parents']
        self.children = entrydict['children']

    def add_tags(self, tags):
        _add2list(tags, self.tags)

    def add_parents(self, parents):
        _add2list(parents, self.parents)

    def add_children(self, children):
        _add2list(children, self.children)

    def add_original_file(self, filep):
        orig_dir = self.path + '/original'
        if isinstance(filep, str):
            filep = [filep]
        self.load_originals()
        hashs = {}
        for iorig in self.original_file:
            rf = open(iorig, 'r')
            hashs[iorig] = hashlib.sha224(rf.read()).hexdigest()
            rf.close()

        for ifile in filep:
            assert (os.path.isfile(ifile))
            rf = open(ifile, 'r')
            hash_ifile = hashlib.sha224(rf.read()).hexdigest()

            if hash_ifile in hashs.values():
                continue

            if ifile not in self.original_file:
                if not os.path.isdir(orig_dir):
                    os.mkdir(orig_dir)

                if not os.path.isfile(orig_dir + '/' + os.path.basename(ifile)):
                    _shutil.copy2(ifile, orig_dir)
                else:
                    i = 0
                    while True:
                        newfile = ifile + '_' + str(i)
                        if not os.path.isfile(orig_dir + '/' + os.path.basename(newfile)):
                            _shutil.copy(ifile, orig_dir + '/' + os.path.basename(newfile))
                            break
                        else:
                            i += 1
        self.load_originals()

    def __str__(self):
        ret = 'Structure: \n' + str(self.structure)
        ret += '\nTags: ' + str(self.tags)
        ret += '\nParents: ' + str(self.parents)
        ret += '\nChildren: ' + str(self.children)
        ret += '\nIdentifier: ' + str(self.identifier)
        ret += '\nOriginal Files:' + str(self.original_file)
        ret += '\n'
        return ret

    def __eq__(self, other):
        ret = True
        if self.structure != other.structure:
            print('Not equal structure')
            ret = False
        elif self.children is None and other.children is not None:
            ret = False
        elif self.children is not None and other.children is None:
            ret = False
        elif self.children is not None and set(self.children) != set(other.children):
            print('Not equal children')
            ret = False
        elif self.parents is None and other.parents is not None:
            ret = False
        elif self.parents is not None and other.parents is None:
            ret = False
        elif self.parents is not None and set(self.parents) != set(other.parents):
            print('Not equal parents')
            ret = False
        elif self.tags is None and other.tags is not None:
            ret = False
        elif self.tags is not None and other.tags is None:
            ret = False
        elif self.tags is not None and set(self.tags) != set(other.tags):
            print('Not equal tags')
            ret = False
        return ret

    def __ne__(self, other):
        return not self.__eq__(other)


class PropertiesEntry:
    """
    Defines one calc in the Execution Repository
    """

    def __init__(self, structure_entry):
        """
        Creates a new calc repository
        """
        self.entry = structure_entry
        self.properties = {}

    def add_property(self, name, values):
        self.properties[name] = values

    def save(self):
        """
        Save an existing repository information
        """
        wf = open(self.entry.path + '/properties.json', 'w')
        _json.dump(self.properties, wf, sort_keys=True, indent=4, separators=(',', ': '))
        wf.close()

    def load(self):
        """
        Loads an existing db from its configuration file
        """
        rf = open(self.path + '/properties.json', 'r')
        self.properties = deep_unicode(_json.load(rf))
        rf.close()


class StructureRepository:
    """
    Defines the location of the executions repository
    and structure repository and methods to add, remove
    and check those db
    """

    def __init__(self, path):
        """
        Creates new db for calculations and structures

        Args:
        path: (string) Directory path for the structure repository
        """
        self.path = os.path.abspath(path)

        if os.path.isfile(self.path + '/db.json'):
            self.load()
        else:
            self.tags = {}

            if os.path.lexists(self.path):
                if not os.path.isdir(self.path):
                    raise ValueError('Path exists already and it is not a directory')
            else:
                os.mkdir(self.path)
            self.save()

    def todict(self):
        """
        Serialize the values of the db into a dictionary
        """
        repos_dict = {'tags': self.tags}

        return repos_dict

    def fromdict(self, repos_dict):
        self.tags = repos_dict['tags']

    def save(self):
        """
        Save an existing repository information
        """
        wf = open(self.path + '/db.json', 'w')
        _json.dump(self.todict(), wf, sort_keys=True, indent=4, separators=(',', ': '))
        wf.close()

    def load(self):
        """
        Loads an existing db from its configuration file
        """
        rf = open(self.path + '/db.json', 'r')
        try:
            jsonload = deep_unicode(_json.load(rf))
        except ValueError:
            print("Error deserializing the object")
            jsonload = {'tags': {}}
        self.fromdict(jsonload)
        rf.close()

    def rebuild(self):
        ids = self.get_all_entries
        self.tags = {}
        for ident in ids:
            struct_entry = StructureEntry(identifier=ident, repository=self)
            for i in struct_entry.tags:
                if i in self.tags:
                    self.tags[i].append(ident)
                else:
                    self.tags[i] = [ident]
        self.save()

    @property
    def get_all_entries(self):
        return [x for x in os.listdir(self.path) if os.path.isfile(self.path + '/' + x + '/metadata.json')]

    def __len__(self):
        return len(self.get_all_entries)

    def get_formulas(self):
        formulas = {}
        for i in self.get_all_entries:
            ientry = StructureEntry(repository=self, identifier=i)
            formula = ientry.structure.formula
            if formula in formulas:
                formulas[formula].append(i)
            else:
                formulas[formula] = [i]
        return formulas

    def merge2entries(self, orig, dest):
        assert (orig.structure == dest.structure)
        dest.add_parents(orig.parents)
        dest.add_children(orig.children)
        dest.add_tags(orig.tags)
        if orig.original_file is not None and len(orig.original_file) > 0:
            dest.add_original_file(orig.original_file)
        dest.save()
        self.del_entry(orig)

    def clean(self):
        for i in self.tags:
            for j in self.tags[i]:
                if not os.path.isdir(self.path + '/' + j) or not os.path.isfile(self.path + '/' + j + '/metadata.json'):
                    print('Removing', j)
                    self.tags[i].remove(j)
        self.save()

    def refine(self):
        formulas = self.get_formulas()
        for j in formulas:
            print(j)
            if len(formulas[j]) > 1:
                for i in range(len(formulas[j]) - 1):
                    stru1 = StructureEntry(repository=self, identifier=formulas[j][i])
                    stru2 = StructureEntry(repository=self, identifier=formulas[j][i + 1])
                    if stru1 == stru2:
                        self.merge2entries(stru1, stru2)
        self.save()

    def merge(self, other):
        """
        Add all the contents from other db into the
        calling object

        :param other: StructureRepository
        """
        conflict_entries = []
        for i in other.get_all_enties:
            if i in self.get_all_entries:
                other_structure = StructureEntry(repository=other, identifier=i)
                this_structure = StructureEntry(repository=self, identifier=i)
                if this_structure != other_structure:
                    conflict_entries.append(i)
        if len(conflict_entries) == 0:
            for i in other.get_all_enties:
                if i not in self.get_all_entries:
                    _shutil.copytree(other.path + '/' + i, self.path + '/' + i)
        else:
            print('Conflict entries found, No merge done')
            return conflict_entries

    def add_entry(self, entry):
        """
        Add a new StructureEntry into the repository
        """
        entry.repository = self
        entry.path = self.path + '/' + entry.identifier
        if not os.path.isdir(entry.path):
            os.mkdir(entry.path)
        entry.save()
        if entry.tags is not None:
            for itag in entry.tags:
                if itag in self.tags:
                    if entry.identifier not in self.tags[itag]:
                        self.tags[itag].append(entry.identifier)
                else:
                    self.tags[itag] = [entry.identifier]
        self.save()

    def add_many_entries(self, list_of_entries, tag, number_threads=1):

        from threading import Thread
        from pychemia.external.pymatgen import cif2structure

        def worker(cifs, tags, results):
            results['succeed'] = []
            results['failed'] = []
            for icif in cifs:
                try:
                    struct = cif2structure(icif, primitive=True)
                except ValueError:
                    struct = None
                    results['failed'].append(icif)
                if struct is not None:
                    structentry = StructureEntry(structure=struct, original_file=icif, tags=[tags])
                    self.add_entry(structentry)
                    results['succeed'].append(icif)

        th = []
        result_list = []
        num = int(math.ceil(float(len(list_of_entries)) / number_threads))
        for i in range(number_threads):
            result_list.append({})
            th.append(Thread(target=worker,
                             args=(
                                 list_of_entries[i * num:min((i + 1) * num, len(list_of_entries))], tag,
                                 result_list[i])))
        for i in th:
            i.start()
        return th, result_list

    def del_entry(self, entry):
        print('Deleting ', entry.identifier)
        for i in entry.tags:
            self.tags[i].remove(entry.identifier)
        _shutil.rmtree(entry.path)

    def __str__(self):
        ret = 'Location: ' + self.path
        ret += '\nNumber of entries: ' + str(len(self))
        if len(self.tags) > 0:
            for itag in self.tags:
                ret += '\n\t' + itag + ':'
                ret += '\n' + str(self.tags[itag])
        else:
            ret += '\nTags: ' + str(self.tags)
        return ret

    def structure_entry(self, ident):
        return StructureEntry(repository=self, identifier=ident)


class ExecutionRepository:
    """
    Defines the location and properties of the Repository
    where all the executions will be stored
    """

    def __init__(self):
        """
        Creates a Repository for Executions
        """
        pass


def _add2list(orig, dest):
    if isinstance(orig, str):
        if orig not in dest:
            dest.append(orig)
    elif isinstance(orig, list):
        for iorig in dest:
            if iorig not in dest:
                dest.append(iorig)



from ...tasks import Task
import os
import shutil
import numpy as np
from ..kpoints import write_kpoints
from ..poscar import write_poscar
from ..incar import write_incar
from ..input import VaspInput
from ....runner import Runner

__author__ = 'Guillermo Avendano-Franco'


class Polarization(Task):
    def report(self):
        pass

    def save(self, filename=None):
        pass

    def plot(self):
        pass

    def status(self):
        pass

    def load(self):
        pass

    def __init__(self, structure, workdir, potcar_filepath, external=None, maxfield=0, stepfield=0, executable='vasp'):

        self.external = None
        self.potcar = potcar_filepath

        if external is not None and external in ['electric', 'magnetic']:
            self.external = external

        self.maxfield = abs(maxfield)
        self.stepfield = abs(stepfield)
        task_params = {'maxfield': self.maxfield, 'stepfield': self.stepfield, 'external': self.external,
                       'portcar': self.potcar}
        Task.__init__(self, structure=structure, task_params=task_params, workdir=workdir, executable=executable)

    def initialize(self, kpoints, cleandir=False):
        if not os.path.isdir(self.workdir):
            os.mkdir(self.workdir)
        elif cleandir:
            shutil.rmtree(self.workdir)
            os.mkdir(self.workdir)

        if self.maxfield > 0:
            signs = ['+', '-']
            fields = np.arange(0, self.maxfield, self.stepfield)
        else:
            signs = ['']
            fields = [0]

        for sign in signs:
            for field in fields:
                pathname = self.workdir + os.sep + 'Field' + sign + str(field)
                if not os.path.isdir(pathname):
                    os.mkdir(pathname)
                for step in ['SCF', 'BANDS', 'berry_IGPAR1', 'berry_IGPAR2', 'berry_IGPAR3']:
                    if not os.path.isdir(pathname + os.sep + step):
                        os.mkdir(pathname + os.sep + step)
                    write_kpoints(kpoints, pathname + os.sep + step + os.sep + 'KPOINTS')
                    if not os.path.lexists(pathname + os.sep + step + os.sep + 'POTCAR'):
                        os.symlink(os.path.abspath(self.potcar), pathname + os.sep + step + os.sep + 'POTCAR')

    def run(self, mode='local'):

        if self.maxfield > 0:
            signs = ['+', '-']
            fields = np.arange(0, self.maxfield, self.stepfield)
        else:
            signs = ['']
            fields = [0]

        success = True
        for sign in signs:
            for field in fields:
                pathname = self.workdir + os.sep + 'Field' + sign + str(field)
                for step in ['SCF', 'BANDS', 'berry_IGPAR1', 'berry_IGPAR2', 'berry_IGPAR3']:
                    tk = Task()
                    tk.minimum(ENCUT=200, POTCAR=self.potcar)
                    tk.vaspinput['EDIFF'] = 1E-9
                    tk.vaspinput['IBRION'] = -1
                    tk.vaspinput['NSW'] = 0
                    if step == 'SCF':
                        iv = VaspInput(variables=tk.vaspinput)
                        write_incar(iv, pathname + os.sep + step + os.sep + 'INCAR')
                        write_poscar(structure=self.structure, filepath=pathname + os.sep + step + os.sep + 'POSCAR')
                        options = {'nproc': 4, 'code_bin': '/home/guilleaf/local/src/vasp.5.3/vasp', 'mpi': True}
                        runner = Runner('vasp', 'local', options)
                        runner.run(dirpath=pathname + os.sep + step)
                        rf = open(pathname + os.sep + step + os.sep + 'vasp.stdout')
                        if rf.readlines()[-2].strip() != 'writing wavefunctions':
                            print('Finished writing the Wavefunctions')
                            success = True
                        else:
                            success = False
                    if step == 'BANDS':
                        shutil.copy('KPOINTS-path', pathname + os.sep + step + os.sep + 'KPOINTS')
                        shutil.copy2(pathname + os.sep + 'SCF/CHGCAR', pathname + os.sep + step)
                        tk.vaspinput['ICHARG'] = 11
                        iv = VaspInput(variables=tk.vaspinput)
                        write_incar(iv, pathname + os.sep + step + os.sep + 'INCAR')
                        write_poscar(structure=self.structure, filepath=pathname + os.sep + step + os.sep + 'POSCAR')
                        options = {'nproc': 4, 'code_bin': 'vasp', 'mpi': True}
                        runner = Runner('vasp', 'local', options)
                        runner.run(dirpath=pathname + os.sep + step)
                        rf = open(pathname + os.sep + step + os.sep + 'vasp.stdout')
                        if rf.readlines()[-2].strip() != 'writing wavefunctions':
                            print('Finished writing the Wavefunctions')
                            success = True
                        else:
                            success = False
                    if step.startswith('berry_IGPAR'):
                        shutil.copy2(pathname + os.sep + 'SCF/CHGCAR', pathname + os.sep + step)
                        tk.vaspinput['ICHARG'] = 1
                        tk.vaspinput['EDIFF'] = 1E-9
                        tk.vaspinput['IBRION'] = -1
                        tk.vaspinput['NSW'] = 0
                        tk.vaspinput['LBERRY'] = True
                        tk.vaspinput['NPPSTR'] = 12
                        tk.vaspinput['IGPAR'] = int(step[-1])
                        if 'ISIF' in tk.vaspinput:
                            tk.vaspinput.pop('ISIF')

                        iv = VaspInput(variables=tk.vaspinput)
                        write_incar(iv, pathname + os.sep + step + os.sep + 'INCAR')
                        write_poscar(structure=self.structure, filepath=pathname + os.sep + step + os.sep + 'POSCAR')
                        options = {'nproc': 4, 'code_bin': '/home/guilleaf/local/src/vasp.5.3/vasp', 'mpi': True}
                        runner = Runner('vasp', 'local', options)
                        runner.run(dirpath=pathname + os.sep + step)
                        rf = open(pathname + os.sep + step + os.sep + 'vasp.stdout')
                        if rf.readlines()[-2].strip() == 'writing wavefunctions':
                            print('Finished writing the Wavefunctions')
                            success = True
                        else:
                            success = False
        return success

    def postprocess(self):
        pass



#!D:\bld\pychemia_1604228883768\_h_env\python.exe

import os
import argparse
import numpy as np
import matplotlib
matplotlib.use('agg')
import matplotlib.pyplot as plt
plt.rcdefaults()
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import matplotlib.patches as mpatches
from matplotlib.collections import PatchCollection
import pychemia
import bson


def label(xy, text, shiftx):
    px = xy[0] - shiftx
    py = xy[1] - 0.0
    plt.text(px, py, text, va='center', ha="center", family='sans-serif', size=8)


def compare_params(path):

    abinitout = popu.get_final_abinit_out(path)

    if not os.path.isfile(path+os.sep+'abinit.in'):
        print('ERROR: No abinit.in found at %s' % path)
        return

    # For making easier to see the values
    np.set_printoptions(linewidth=200, suppress=True)

    # Reading the INPUT
    abi = pychemia.code.abinit.AbinitInput(path+os.sep+'abinit.in')
    idmatpawu = np.array(abi['dmatpawu']).reshape(-1, 5, 5)
    iparams = pychemia.population.orbitaldftu.dmatpawu2params(idmatpawu, 5)

    # Reading the OUTPUT
    dmatpawu = pychemia.population.orbitaldftu.get_final_dmatpawu(abinitout)
    odmatpawu = np.array(dmatpawu).reshape(-1, 5, 5)
    oparams = pychemia.population.orbitaldftu.dmatpawu2params(odmatpawu, 5)

    print('PARAMETRIC REPRESENTATION found at %s' % abinitout)
    for i in sorted(list(iparams.keys())):
        print(i)
        print('input')
        print(iparams[i])
        print('output')
        print(i)
        print(oparams[i])

    abo = pychemia.code.abinit.AbinitOutput(abinitout)
    if not abo.is_finished:
        print('This output is not finished')
    try:
        nres2 = abo.get_energetics()['nres2'][-1]
        etot = abo.get_energetics()['etot'][-1]
        nscf = len(abo.get_energetics()['etot'])
        print("%30s ETOT: %15.6f NRES2: %15.6e NumSCF: %3d" % (path, etot, nres2, nscf))
    except:
        print("ERROR: Could not get energetics from %s" % abinitout)


def create_population():
    popu.random_population(64)


def prepare_folders(scrpath):
    for i in popu.members:
        popu.prepare_folder(i, workdir=scrpath, source_dir=scrpath)


def check_status(basepath, dirs):

    print("%-40s %15s %15s %4s" % ("ABINIT output", "ETOT", 'nres2', 'nSCF'))
    for i in dirs:
        path = basepath+os.sep+i
        abinitout = popu.get_final_abinit_out(path)
        if abinitout is None:
            continue

        abo = pychemia.code.abinit.AbinitOutput(abinitout)
        if not abo.is_finished:
            continue
        try:
            nres2 = abo.get_energetics()['nres2'][-1]
            etot = abo.get_energetics()['etot'][-1]
            nscf = len(abo.get_energetics()['etot'])
            print("%-40s %15.6f %15.6e %4d" % (abinitout, etot, nres2, nscf))
        except:
            print("ERROR: Could not get final energetics from %s" % abinitout)


def plot_status(basepath, dirs, nstep=50):
    print('Plotting Status...')
    X = {}
    Y = {}

    # Get the data:
    for path in dirs:
        X[path] = np.array([])
        Y[path] = {}
        Y[path]['nres2'] = np.array([])
        Y[path]['etot'] = np.array([])
        Y[path]['delta'] = np.array([])

        for i in range(10):
            if os.path.isfile(basepath+os.sep+path+os.sep+'abinit_%d.out' % i):
                abo = pychemia.code.abinit.AbinitOutput(basepath+os.sep+path+os.sep+'abinit_%d.out' % i)
                if not abo.is_finished:
                    print('This output is not finished')
                try:
                    energ = abo.get_energetics()
                    nres2 = energ['nres2'][-1]
                    etot = energ['etot'][-1]
                    nscf = len(energ['etot'])
                    x = np.arange(nstep*i, nstep*i + len(energ['nres2']))
                    yetot = np.array(energ['etot'])
                    ynres2 = np.array(energ['nres2'])
                    ydelta = np.array(np.abs(energ['deltaEh']))
                    X[path] = np.concatenate((X[path], x))
                    Y[path]['etot'] = np.concatenate((Y[path]['etot'], yetot))
                    Y[path]['nres2'] = np.concatenate((Y[path]['nres2'], ynres2))
                    Y[path]['delta'] = np.concatenate((Y[path]['delta'], ydelta))
                    print("%s ETOT:%15.6f NRES2=%15.6e Num SCF=%3d" % (path, etot, nres2, nscf))
                except:
                    print("%s Failed processing output" % path)

    # RESIDUAL
    plt.figure(figsize=(8, 11))
    plt.subplots_adjust(left=0.1, bottom=0.05, right=0.99, top=0.99,
                        wspace=None, hspace=None)
    for path in dirs:
        if len(X) > 0:
            plt.semilogy(X[path], Y[path]['nres2'], label=path[-4:])

    for i in nstep*np.arange(10):
        plt.semilogy([i, i], [1E-19, 1E5], '0.5')

    plt.ylim(1E-19, 1E5)
    plt.xlim(0, max([max(X[path]) for path in dirs]))
    plt.legend()
    plt.xlabel("SCF iteration")
    plt.ylabel("Density Residual$^2$")
    plt.savefig('Residual.pdf')

    # ETOT
    plt.figure(figsize=(8, 11))
    for path in dirs:
        if len(X) > 0:
            plt.plot(X[path], Y[path]['etot'], label=path[-4:])

    plt.subplots_adjust(left=0.1, bottom=0.05, right=0.99, top=0.99,
                        wspace=None, hspace=None)

    for i in nstep*np.arange(10):
        plt.plot([i, i], [-992, -990], '0.5')

    plt.ylim(-992, -990)
    plt.xlim(0, max([max(X[path]) for path in dirs]))
    plt.legend()
    plt.xlabel("SCF iteration")
    plt.ylabel("Energy")
    plt.savefig('ETotal.pdf')

    # deltaEh
    plt.figure(figsize=(8, 11))
    for path in dirs:
        if len(X) > 0:
            plt.semilogy(X[path], Y[path]['delta'], label=path[-4:])

    plt.subplots_adjust(left=0.1, bottom=0.05, right=0.99, top=0.99,
                        wspace=None, hspace=None)

    for i in nstep*np.arange(10):
        plt.semilogy([i, i], [1E-13, 1E3], '0.5')

    plt.ylim(1E-13, 1E3)
    plt.xlim(0, max([max(X[path]) for path in dirs]))
    plt.legend()
    plt.xlabel("SCF iteration")
    plt.ylabel("Delta Energy")
    plt.savefig('deltaEh.pdf')


def plot_polar():
    print('Computing duplicates in %d candidates' % len(popu.evaluated))
    dupes = popu.get_duplicates(popu.evaluated)
    print('Number of duplicates    : %d' % len(dupes))
    nodupes = [x for x in popu.evaluated if x not in dupes]
    print('Number of not duplicates: %d' % len(nodupes))

    if candidates is not None:
        nodupes = candidates
    
    fig = plt.figure(figsize=(20, 2.01*len(nodupes)))

    etots = np.array([popu.value(x) for x in nodupes])
    sort_entries = np.array(nodupes)[etots.argsort()]

    N = len(nodupes)
    M = 10

    grid = np.mgrid[0.02:0.87:complex(0, M), 0.02:0.84:complex(0, N)].T
    print("List of duplicates to plot:")
    for i in nodupes:
        print(i)

    for i in range(N):

        entry_id = sort_entries[i]
        pm = popu.get_correlation_params(entry_id)
        etot = popu.value(entry_id)
        ea = np.array(pm['euler_angles'])
        nres2 = popu.get_entry(entry_id, {'properties.nres2': 1})['properties']['nres2']

        fig.text(0.03, grid[i, 0][1]+0.08, "%7.2f" % etot, va='center', ha='center', fontsize='24', rotation='vertical',
                 color='red')
        fig.text(0.01, grid[i, 0][1]+0.08, "%9.2E" % nres2, va='center', ha='center', fontsize='16', rotation='vertical',
                 color='blue')

        radii = np.array([1, 1, 1, 1])
        #colors = np.array([0, 1, 2, 3])
        colors = ['red', 'blue', 'fuchsia', 'yellow']
        width = np.array([0.15, 0.15, 0.1, 0.1])

        for j in range(M):

            theta = ea[:, j]

            ax = fig.add_axes([grid[i, j][0], grid[i, j][1], 0.146, 0.146], projection='polar')
            ax.set_axis_bgcolor('mintcream')
            #ax.yaxis.set_tick_params(labelsize=0)
            #ax.xaxis.set_tick_params(labelsize=0)
            ax.get_xaxis().set_visible(False)
            ax.get_yaxis().set_visible(False)
            ax.spines['polar'].set_visible(True)
            bars = ax.bar(theta, radii, width=width, bottom=0.0, linewidth=0)

            # Use custom colors and opacity
            for r, bar in zip(colors, bars):
                #bar.set_facecolor(plt.cm.Paired(r))
                bar.set_facecolor(r)
                bar.set_alpha(0.9)

    plt.subplots_adjust(left=0, right=1, bottom=0, top=1)
    #plt.axis('equal')
    #plt.axis('off')
    plt.savefig(figname+'_polar.png')


def plot_evolution():
    N = ff.current_generation
    M = ff.generation_size

    fig, ax = plt.subplots(figsize=(20, 20))
    grid = np.mgrid[0.1:0.9:complex(0, N), 0.1:0.9:complex(0, M)].T

    patches = []
    colors = []

    line_color = {'replace_by_other': 'green', 'replace_by_random': 'cyan', 'promoted': 'black', 'duplicate': 'orange'}
    dx = 0.01

    for j in range(N):
        dup_dx = 0.5 * dx
        for i in range(M):
            entry_id = ff.lineage[str(i)][j]

            nres2 = ff.population.get_entry(entry_id, {'properties.nres2': 1})['properties']['nres2']
            if nres2 < 1E14:
                lw = 1
                ls = 'solid'
            else:
                lw = 1
                ls = 'dotted'

            if ff.population.is_evaluated(entry_id):
                colors.append(ff.population.value(entry_id))
                circle = mpatches.Circle(grid[i, j], 0.4 / float(M), ec="black", linewidth=lw, linestyle=ls)
                patches.append(circle)
                label(grid[i, j], "%7.2f" % ff.population.value(entry_id), 0.0)

            for ichange in ff.population.pcdb.db.generation_changes.find({'from': entry_id, 'generation': j}):
                if ichange['change'] == 'duplicate':
                    orig = ichange['from']
                    dest = ichange['to']
                    newi = int(ff.lineage_inv[dest])
                    dup_dx += dx/10.0
                    x, y = np.array([[grid[i, j][0] - 1.5 * dup_dx, grid[i, j][0] - 2 * dup_dx,
                                      grid[newi, j][0] - 2 * dup_dx, grid[newi, j][0] - dx],
                                     [grid[i, j][1], grid[i, j][1], grid[newi, j][1], grid[newi, j][1]]])
                    line = mlines.Line2D(x, y, lw=1., alpha=0.8, color=line_color[ichange['change']],
                                         marker='>', markersize=5, markeredgecolor='none')
                    line.set_markevery([3])
                    ax.add_line(line)
                elif j < N - 1:
                    x, y = np.array(
                        [[grid[i, j][0] + dx, grid[i, j + 1][0] - 2 * dx], [grid[i, j][1], grid[i, j + 1][1]]])
                    line = mlines.Line2D(x, y, lw=5., alpha=0.3, color=line_color[ichange['change']])
                    # label(0.5*(grid[i, j]+grid[i, j+1]), ichange['change'], 0.0)
                    ax.add_line(line)

    collection = PatchCollection(patches, cmap=plt.cm.hsv, alpha=0.3)
    collection.set_array(np.array(colors))
    ax.add_collection(collection)

    plt.subplots_adjust(left=0, right=1, bottom=0, top=1)
    plt.axis('equal')
    plt.axis('off')
    plt.savefig(figname+'_evo.pdf')


if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='Orbital DFTU Plotting tool')

    parser.add_argument('-host', type=str, help='Hostname or IP of the Mongo Server', required=True)
    parser.add_argument('-name', type=str, help='Name of Database', required=True)
    parser.add_argument('-dbuser', type=str, help='Username on Database', required=True)
    parser.add_argument('-figname', type=str, help='Path to output figure', default=None)
    parser.add_argument('-basepath', type=str, help='Path where calculations are performed', default='.')
    parser.add_argument('-candidates', type=str, help='IDs for candidates', nargs='+')

    args = parser.parse_args()
    basepath = args.basepath

    if args.figname is None:
        figname = args.name
    else:
        figname = args.name

    if figname[-4] == '.':
        figname = figname[:-4]

    passwd = input('Password:')

    db_settings = {'name': args.name, 'host': args.host, 'user': args.dbuser, 'passwd': passwd}
    pcdb = pychemia.db.get_database(db_settings)
    popu = pychemia.population.OrbitalDFTU(pcdb, basepath+'/abinit.in')
    popu.recover()

    ff = pychemia.searcher.FireFly(popu)
    ff.recover()
    print(ff)

    if len(args.candidates) > 0:
        candidates = [bson.ObjectId(x) for x in args.candidates]
    else:
        candidates = None

    for i in candidates:
        assert(popu.get_entry(i) is not None)
        entry = popu.get_entry(i)
        print("%s %f" % (entry['_id'], entry['properties']['etot']))
        
    print('Plotting Evolution')
    plot_evolution()
    print('Plotting Euler Angles')
    plot_polar()



import os
import math
from numbers import Number
import numpy as np
from pychemia.utils.computing import deep_unicode
from ..codes import CodeInput


class VaspInput(CodeInput):
    """
    VASP INCAR object

    It contains:

    data:
          variables = Dictionary whose keys are VASP variable names
                      and contains the values as numpy arrays

    methods:
            write = Write the input into as a text file that VASP
                    can use as an input file

            get_value = Get the value of a particular variable
            set_value = Set the value of a particular variable
    """
    def __init__(self, filename=None, variables=None):

        # CodeInput.__init__(self)
        if variables is not None:
            for i in variables:
                self.__dict__[i] = variables[i]

        if filename is not None and os.path.isfile(filename):
            try:
                self.__import_input(filename)
            except ValueError:
                print('File format not identified')

    def _clean_variables(self):
        for i in self.__dict__:
            value = self.__dict__[i]
            if isinstance(value, list):
                if len(value) == 1:
                    self.__dict__[i] = value[0]

    @property
    def variables(self):
        return self.__dict__

    def __import_input(self, filename):
        rf = open(filename, 'r')
        for line in rf.readlines():
            line = line.partition('#')[0]
            line = line.rstrip()
            if '=' in line:
                varname = line.split('=')[0].strip().upper()
                value = line.split('=')[1].strip()
                if value[-1] == ';':
                    value = value[:-1]
                if varname == '$SYSTEM':
                    self.__dict__[varname] = value
                elif value.upper() == '.FALSE.':
                    self.__dict__[varname] = False
                elif value.upper() == '.TRUE.':
                    self.__dict__[varname] = True
                else:
                    try:
                        self.__dict__[varname] = int(value)
                    except ValueError:
                        try:
                            self.__dict__[varname] = round(float(value), 10)
                        except ValueError:
                            self.__dict__[varname] = self._deep_parsing(value)
        rf.close()
        self._clean_variables()

    @staticmethod
    def _deep_parsing(value):

        # Try splitting
        val_splt = value.split()
        ret = []
        if len(val_splt) == 1:
            ret = value
        else:
            for i in range(len(val_splt)):
                try:
                    newval = [int(val_splt[i])]
                except ValueError:
                    try:
                        newval = [round(float(val_splt[i]), 10)]
                    except ValueError:
                        if '*' in val_splt[i] and len(val_splt[i].split('*')) == 2:
                            # print 'Trying this', val_splt[i]
                            newval = int(val_splt[i].split('*')[0]) * [float(val_splt[i].split('*')[1])]
                        else:
                            newval = [val_splt[i]]
                ret += newval
        return ret

    def read(self, filename='INCAR'):
        pass

    def write(self, filename='INCAR'):
        """
        Write an inputvars object into a text
        file that VASP can use as an INCAR
        file

        Args:
            filename:
                The path to 'INCAR' filename that will be written
        """
        wf = open(filename, 'w')
        wf.write(self.__str__())
        wf.close()

    def write_key(self, varname):
        """
        Receives an input variable and write their contents
        properly according with their kind and length

        Args:
            varname:
                The name of the input variable
        """
        ret = (varname.ljust(15)) + " =  "
        if varname not in self.__dict__:
            raise ValueError("[ERROR] input variable: '%s' is not declared" % varname)

        value = self.__dict__[varname]
        value = deep_unicode(value)
        if isinstance(value, bool):
            if value:
                ret += '.TRUE.'
            else:
                ret += '.FALSE.'
        elif isinstance(value, Number):
            ret += str(value)
        elif isinstance(value, str):
            ret += value
        else:
            # Assume that the variables are integer and test if such assumption
            # is true
            integer = True
            real = False
            string = False
            compact = True

            # Get the general kind of values for the input variable
            for j in self.__dict__[varname]:

                try:
                    if not float(j).is_integer():
                        # This is the case of non integer values
                        integer = False
                        real = True
                        string = False
                        if len(str(float(j))) > 7:
                            compact = False

                except ValueError:
                    # This is the case of '*1' that could not
                    # be converted because we do not know the size
                    # of the array
                    integer = False
                    real = False
                    string = True

            if len(self.__dict__[varname]) == 1:
                ret += "%s" % self.__dict__[varname][0]
            if len(self.__dict__[varname]) > 1:
                tmp = [(self.__dict__[varname][0], 1)]
                prev = self.__dict__[varname][0]
                for i in self.__dict__[varname][1:]:
                    if i == prev:
                        tmp[-1] = (i, tmp[-1][1] + 1)
                    else:
                        tmp.append((i, 1))
                    prev = i

                counter = 0
                for j in tmp:

                    if j[1] > 3:
                        if real:
                            if compact:
                                ret += (" %d*%g" % (j[1] - j[1] % 3, j[0])).rjust(8)
                            else:
                                ret += " %d*%g" % (j[1] - j[1] % 3, j[0])
                        elif integer:
                            ret += " %d*%d" % (j[1] - j[1] % 3, j[0])
                        else:
                            ret += " %d*%s" % (j[1] - j[1] % 3, j[0])

                        if j[1] % 3 != 0:
                            for i in range(j[1] % 3):
                                if real:
                                    if compact:
                                        ret += (" %g" % j[0]).rjust(8)
                                    else:
                                        ret += " %17.10e" % j[0]
                                elif integer:
                                    ret += " %d" % j[0]
                                else:
                                    ret += " %s" % j[0]

                        counter += j[1]
                    else:
                        for i in range(j[1]):
                            if real:
                                if compact:
                                    ret += (" %g" % j[0]).rjust(8)
                                else:
                                    ret += " %17.10e" % j[0]
                            elif integer:
                                ret += " %d" % j[0]
                            elif string:
                                ret += " %s" % j[0]

                            counter += 1

        ret += ";\n"
        return ret

    def set_encut(self, ENCUT=300, POTCAR=None):
        self.__dict__['ENCUT'] = ENCUT
        if POTCAR is not None and ENCUT < 10:
            maxvalue = 0
            if not os.path.isfile(POTCAR):
                raise ValueError('Not such file', POTCAR)
            rf = open(POTCAR)
            for line in rf.readlines():
                if 'ENMAX' in line:
                    list4line = line.split()
                    assert (list4line[0].strip() == 'ENMAX')
                    value = list4line[2].strip()
                    if value[-1] == ';':
                        value = value[:-1]
                    value = float(value)
                    if value > maxvalue:
                        maxvalue = value
            rf.close()
            if ENCUT < 10:
                self.__dict__['ENCUT'] = int(math.ceil(ENCUT * maxvalue))
            else:
                self.__dict__['ENCUT'] = maxvalue
                # pcm_log.debug('ENCUT: %7.3f' % self.variables['ENCUT'])

    def set_ismear(self, kpoints):
        if np.prod(kpoints.grid) > 27:
            self.__dict__['ISMEAR'] = -5
        else:
            self.__dict__['ISMEAR'] = 0

    def set_ion_relax(self, NSW=50, ISIF=2, IBRION=2, EDIFFG=-1E-3):
        self.__dict__['IBRION'] = IBRION
        self.__dict__['NSW'] = NSW
        self.__dict__['ISIF'] = ISIF
        self.__dict__['EDIFFG'] = EDIFFG

    def set_electron_scf(self, NELM=60, NELMIN=2, EDIFF=1E-4, IALGO=48):
        self.__dict__['NELMIN'] = NELMIN
        self.__dict__['NELM'] = NELM
        self.__dict__['EDIFF'] = EDIFF
        self.__dict__['IALGO'] = IALGO

    def set_rough_relaxation(self):
        self.set_minimum(PREC='Normal', ISPIN=1, LREAL=False, ISMEAR=0, LORBIT=11)
        self.set_electron_scf(NELM=60, NELMIN=2, EDIFF=1E-4, IALGO=48)
        self.set_ion_relax(NSW=50, ISIF=2, IBRION=2, EDIFFG=-1E-2)
        # self.__dict__['NPAR'] = 2

    def set_mit_settings(self):
        self.set_minimum(PREC='Accurate', ISPIN=2, LREAL=False, ISMEAR=-5, LORBIT=11)
        self.set_electron_scf(NELM=100, NELMIN=6, EDIFF=5E-5, IALGO=48)
        self.set_ion_relax(NSW=99, ISIF=3, IBRION=2, EDIFFG=-1E-3)
        self.__dict__['LWAVE'] = False
        self.__dict__['SIGMA'] = 0.05
        self.__dict__['LDAU'] = True
        self.__dict__['LDAUTYPE'] = 2
        self.__dict__['ICHARG'] = 1

    def set_minimum(self, PREC='Normal', ISPIN=2, LREAL=False, ISMEAR=0, LORBIT=11):
        self.__dict__['PREC'] = PREC
        if LREAL is not None:
            self.__dict__['LREAL'] = LREAL
        else:
            self.__dict__['LREAL'] = 'Auto'
        self.__dict__['ISMEAR'] = ISMEAR
        self.__dict__['ISPIN'] = ISPIN
        self.__dict__['LORBIT'] = LORBIT
        # self.__dict__['NPAR'] = 2
        self.__dict__['LASPH'] = True

    def set_density_for_restart(self):
        self.__dict__['ICHARG'] = 1

    def get_defined_variables(self):
        return list(self.keys())

    # The next five methods are requirements of the ABC.
    def __setitem__(self, key, value):
        self.__dict__[key] = value

    def __getitem__(self, key):
        return self.__dict__[key]

    def __delitem__(self, key):
        del self.__dict__[key]

    def __iter__(self):
        return iter(self.__dict__)

    def __len__(self):
        return len(self.__dict__)

    # The final two methods aren't required, but nice for demo purposes:
    def __str__(self):
        """
        returns simple dict representation of the mapping
        """
        ret = ''
        for i in sorted(self.__dict__.keys()):
            ret += self.write_key(i)
        return ret

    def __repr__(self):
        """
        echoes class, id, & reproducible representation in the REPL
        """
        return '{}, {}(variables={})'.format(super(self.__class__, self).__repr__(),
                                             self.__class__.__name__,
                                             self.__dict__)






import json
import os

import bson
import networkx as nx
from networkx.readwrite import json_graph


class NetworkAnalysis:
    def __init__(self, population, tolerance=0.1):
        self.population = population
        self.tolerance = tolerance
        self.pcdb = self.population.pcdb

    def get_network(self):

        # print 'Population evaluated', len(self.population.evaluated)
        lista = list(self.population.ids_sorted(self.population.evaluated))

        graph = nx.Graph()

        for change in self.pcdb.db.generation_changes.find({'change': 'replace_by_other'}):
            graph.add_edge(str(change['from']), str(change['to']))

        for change in self.pcdb.db.generation_changes.find({'change': 'duplicate'}):
            graph.add_edge(str(change['from']), str(change['to']))

        # print 'Number of nodes:', graph.number_of_nodes()
        # print 'Number of edges:', graph.number_of_edges()

        for n in graph:
            graph.node[n]['name'] = n

        for n in graph:
            entry_id = bson.ObjectId(n)
            if 'spacegroup' in self.population.get_entry(bson.ObjectId(n))['properties']:
                graph.node[n]['group'] = str(self.population.get_entry(bson.ObjectId(n))['properties']['spacegroup'])
                graph.node[n]['name'] = graph.node[n]['group']
            else:
                graph.node[n]['group'] = str(20 * lista.index(entry_id) / len(lista))
                graph.node[n]['name'] = str(lista.index(entry_id))

        # print 'Computing distances...'
        for n in graph.edges():
            graph[n[0]][n[1]]['value'] = int(10 * self.population.distance(bson.ObjectId(n[0]), bson.ObjectId(n[1])))
        # print 'done'

        d = json_graph.node_link_data(graph)
        json.dump(d, open('Network_%s.json' % self.population.name, 'w'), sort_keys=True, indent=4,
                  separators=(',', ': '))

        if not os.path.isfile('NetworkBasins_%s.json' % self.population.name):
            dupes_dict, dupes_list = self.population.dict_duplicates(self.population.evaluated, fast=True)
            new_dupes_dict = {}
            for i in dupes_dict:
                new_dupes_dict[str(i)] = []
                for j in dupes_dict[i]:
                    new_dupes_dict[str(i)].append(str(j))

            wf = open('NetworkBasins_%s.json' % self.population.name, 'w')
            json.dump(new_dupes_dict, wf, indent=2, separators=(',', ': '))
            wf.close()
        else:
            rf = open('NetworkBasins_%s.json' % self.population.name, 'r')
            new_dupes_dict = json.load(rf)

        # print 'Number of non-duplicates', len(new_dupes_dict)

        # print 'Reverting dictionary...'
        graph_basins = nx.Graph()
        tabla_reversa = {}
        for i in lista:
            if str(i) in new_dupes_dict:
                for j in new_dupes_dict[str(i)]:
                    # print lista.index(bson.ObjectId(i)),' : ', lista.index(bson.ObjectId(j))
                    if j not in tabla_reversa:
                        tabla_reversa[j] = str(i)
        # print 'done'

        for i in graph.edges_iter():
            if i[0] in tabla_reversa and i[1] in tabla_reversa:
                graph_basins.add_edge(tabla_reversa[i[0]], tabla_reversa[i[1]])

        # print 'Number of nodes:', graph_basins.number_of_nodes()
        # print 'Number of edges:', graph_basins.number_of_edges()

        for n in graph_basins:
            entry_id = bson.ObjectId(n)
            if 'spacegroup' in self.population.get_entry(entry_id)['properties']:
                spacegroup = str(self.population.get_entry(entry_id)['properties']['spacegroup'])
                graph_basins.node[n]['group'] = spacegroup
                graph_basins.node[n]['name'] = spacegroup
            else:
                graph_basins.node[n]['group'] = str(20 * lista.index(entry_id) / len(lista))
                graph_basins.node[n]['name'] = str(lista.index(entry_id))

        for n in graph_basins.edges():
            graph_basins[n[0]][n[1]]['value'] = int(
                10 * self.population.distance(bson.ObjectId(n[0]), bson.ObjectId(n[1])))

        d2 = json_graph.node_link_data(graph_basins)
        json.dump(d2, open('Network_%s_basins.json' % self.population.name, 'w'), sort_keys=True, indent=4,
                  separators=(',', ': '))

        best = lista[0]
        line = 'Population: %10s Evaluated: %4d   N:%5d E:%5d    BN:%4d BE:%4d  %7.2f  GG: %8s %8s'
        print(line % (self.population.name,
                      len(self.population),
                      graph.number_of_nodes(),
                      graph.number_of_nodes(),
                      graph_basins.number_of_nodes(),
                      graph_basins.number_of_edges(),
                      float(graph_basins.number_of_edges()) / (1.0 + graph_basins.number_of_nodes()),
                      str(best) in graph.nodes(),
                      str(best) in graph_basins.nodes()))



from ..codes import CodeInput
from pychemia.utils.periodic import atomic_symbol
from pychemia import Structure
import os
import numpy as np


class SiestaInput(CodeInput):

    def __init__(self, inputfile=None):
        CodeInput.__init__(self)
        if inputfile is not None:
            if os.path.exists(inputfile):
                self.read(inputfile)

    def read(self, filename):
        if not os.path.isfile(filename):
            raise ValueError('ERROR: File not found: %s' % filename)

        try:
            rf = open(filename)
            data = rf.readlines()
        except UnicodeDecodeError:
            rf = open(filename, encoding='ISO-8859-1')
            data = rf.readlines()

        inblock = False
        ret = {}

        for i in range(len(data)):
#            print(data[i])
            # Ignoring blanks
            if data[i].strip() == '':
                continue
            # Storing a comment line
            elif data[i].strip().startswith('#'):
                comment = data[i].strip()
            # Reading a block
            elif data[i].strip().lower().startswith('%block'):
                blockname = data[i][6:].strip()
                inblock = True
                blockdata = []
            elif data[i].strip().lower().startswith('%endblock'):
                # Store the block of data
                ret['block+'+blockname] = list(blockdata)
                inblock = False
            elif inblock:
                lblock = self.process_line(data[i])
                blockdata.append(lblock)
            else:
                lblock = self.process_line(data[i])
                #print('==> %s' % lblock)
                varname = lblock[0]
                if '_' in varname:
                    varname = varname.replace('_', '')
                if len(lblock) == 2 and isinstance(lblock[1], str):
                    if lblock[1].lower() in ['t', 'true', '.true.', 'yes']:
                        value = True
                    elif lblock[1].lower() in ['f', 'false', '.false.', 'no']:
                        value = False
                    else:
                        value = lblock[1]
                    ret[varname] = value
                elif len(lblock) == 2:
                    ret[varname] = lblock[1]
                elif len(lblock) == 1:
                    ret[varname] = True
                elif len(lblock) > 2 and all(isinstance(x, str) for x in lblock):
                    ret[varname] = data[i].strip()[len(varname):].strip()
                else:
                    ret[varname] = lblock[1:]
        self.variables = ret

    def process_line(self, line):
        # Clean from comments
        if '#' in line:
            data = line.split('#')[0]
        else:
            data = line
        # Line block of data
        lblock = []
        for jtoken in data.split():
            try:
                token = int(jtoken)
                lblock.append(token)
            except ValueError:
                try: 
                    token = float(jtoken)
                    lblock.append(token)
                except ValueError:
                    lblock.append(jtoken)
        return lblock

    def __str__(self):
        ret = ''
        
        if len(self.variables) == 0:
            return ret

        keys = list(self.variables.keys())
        maxlength = max([len(i) for i in keys if not i.startswith('block+')])

        partial_keys = [x for x in keys if x.lower().startswith('system')]
        if len(partial_keys) > 0:
            ret += '# System Variables\n\n'
            # System variables
            for i in partial_keys:
                ret += self.write_one(i, maxlength)
                keys.remove(i)
            ret += '\n'

        partial_keys = [x for x in keys if x.lower().startswith('number')]
        if len(partial_keys) > 0:
            ret += '# Number Variables\n\n'
            # Number variables
            for i in partial_keys:
                ret += self.write_one(i, maxlength)
                keys.remove(i)
            ret += '\n'

        partial_keys = [x for x in keys if x.lower().startswith('save')]
        if len(partial_keys) > 0:
            ret += '# Save Variables\n\n'
            # Save variables
            for i in [x for x in keys if x.lower().startswith('save')]:
                ret += self.write_one(i, maxlength)
                keys.remove(i)
            ret += '\n'

        partial_keys = [x for x in keys if not x.lower().startswith('block+')]
        if len(partial_keys) > 0:
            ret += '# Other variables\n\n'
            # Other variables
            for i in partial_keys:
                ret += self.write_one(i, maxlength)
                keys.remove(i)
            ret += '\n'

        partial_keys = [x for x in keys if x.lower().startswith('block+')]
        if len(partial_keys) > 0:
            ret += '# Block Sections\n\n'
            # Block Sections
            for i in partial_keys:
                ret += self.write_one(i, maxlength) + '\n'
                keys.remove(i)
            ret += '\n'

        return ret

    def write_one(self, key, maxlength):

        if isinstance(self.variables[key], str):
            ret = "%s %s" % (key.ljust(maxlength), self.variables[key])
        elif isinstance(self.variables[key], bool):
            if self.variables[key]:
                ret = "%s %s" % (key.ljust(maxlength), 'T')
            else:
                ret = "%s %s" % (key.ljust(maxlength), 'F')
        elif isinstance(self.variables[key], int):
            ret = "%s %d" % (key.ljust(maxlength), self.variables[key])
        elif isinstance(self.variables[key], float):
            ret = "%s %f" % (key.ljust(maxlength), self.variables[key])
        elif key.startswith('block+'):
            ret = '%' + ('block %s\n' % key[6:])
            for iline in self.variables[key]:
                for jtoken in iline:
                    if isinstance(jtoken, float):
                        ret += " %9.5f" % jtoken
                    elif isinstance(jtoken, int):
                        ret += " %d" % jtoken
                    else:
                        ret += " %s" % jtoken
                ret += '\n'
            ret += '%' + ('endblock %s' % key[6:])
        elif isinstance(self.variables[key], list):
            ret = "%s" % key.ljust(maxlength)
            for i in self.variables[key]:
                ret += " %s" % i
        else:
            raise ValueError("%s %s" % (key, self.variables[key]))
        return ret + '\n'

    def get_structure(self):

        natoms = self.get('NumberOfAtoms')
        nspecies = self.get('NumberOfSpecies')
        chemspecies = self.get('block+ChemicalSpeciesLabel')

        #get lattice constant
        lattice_constant = self.get('LatticeConstant')[0]

        #dot with lattice constant to scale
        cell = np.array(self.get('block+LatticeVectors'),dtype=float)*float(lattice_constant)

        positions =[] #np.zeros(natoms)
        symbols = natoms * [None]

        if self.has_variable('AtomicCoordinatesFormat'):
            atomic_format = self.get('AtomicCoordinatesFormat')
        else:
            atomic_format = 'NotScaledCartesianBohr'

        atomic_coords = self.get('block+AtomicCoordinatesAndAtomicSpecies')

        # index = 0
        # for iline in atomic_coords:
        #     positions[index] = np.array(iline[:3])
        #     for i in chemspecies:
        #         if i[0] == iline[-1]:
        #             symbol = atomic_symbol(i[1])
        #     symbols[index] = symbol
        # index += 1

        #modified a bit
        index = 0
        for iline in atomic_coords:
            positions.append(np.array(iline[:3])) #since we can't set an array element with a sequence in np
            symbols[index] = iline[-1]
            index +=1 

        if atomic_format in ['Fractional', 'ScaledByLatticeVectors']:
            return Structure(symbols=symbols, reduced=positions, cell=cell, periodicity=True)
        elif atomic_format in ['Bohr', 'NotScaledCartesianBohr']:
            return Structure(symbols=symbols, positions=positions, cell=cell, periodicity=True)
        elif atomic_format in ['Ang', 'NotScaledCartesianAng']:
            return Structure(symbols=symbols, positions=positions, cell=cell, periodicity=True)



#!D:\bld\pychemia_1604228883768\_h_env\python.exe

import logging
import os
import sys
import json
import time
import getopt
import pychemia
from pychemia.utils.computing import get_int

logging.basicConfig(level=logging.DEBUG)

try:
    from pychemia.symm import symmetrize
except ImportError:
    def symmetrize(x):
        return x


def cleaner():
    files = ['abinit_LOG', 'abinit_STATUS']
    for j in files:
        for i in os.listdir('.'):
            if i.startswith(j):
                os.remove(i)
    if os.path.exists('abinit.err') and os.path.getsize('abinit.err') == 0:
        os.remove('abinit.err')


def usage(name):
    print("""
NAME
    %s

DESCRIPTION
    ABINIT Run with automatic selection of Norm-Conserving PSPs

OPTIONS

    --help, -h
        Return information on the options and use of this script

    --input, -i <string> (Default: 'abinit.in')
        Structure for which the strength will be computed

    --output, -o <string> (Default: 'pcm_results.json')
        File in JSON format where the results are stored

    --nparal, -n <int> (Default: 2)
        Number of MPI parallel processes for ABINIT

    --binary, -b <string> (Default: 'abinit')
        Path to the ABINIT executable

    --pseudo, -p <string> (Default: 'FHI_LDA')
        Kind and Exchange for Pseudopotentials

""" % os.path.basename(name))


def main(argv):
    try:
        opts, args = getopt.getopt(argv[1:], "hi:o:n:b:p:", ["help", "input=", "output=", "nparal=", "binary=",
                                                             'pseudo='])
    except getopt.GetoptError:
        usage(argv[0])
        sys.exit(2)

    if len(opts) == 0:
        usage(argv[0])
        sys.exit(2)

    # Default Values
    workdir = '.'
    inpu_file = 'abinit.in'
    output_file = 'pcm_results.json'
    nparal = 2
    binary = 'abinit'
    pseudo = 'FHI_LDA'

    for opt, arg in opts:

        if opt in ("-h", "--help"):
            usage(argv[0])
            sys.exit()
        elif opt in ("-i", "--input"):
            inpu_file = arg
        elif opt in ("-o", "--output"):
            output_file = arg
        elif opt in ("-n", "--nparal"):
            nparal = get_int(arg)
        elif opt in ("-b", "--binary"):
            binary = arg
        elif opt in ("-p", "--pseudo"):
            pseudo = arg

    structure = pychemia.code.abinit.AbinitInput(inpu_file).get_structure()
    if structure is None:
        print(" ERROR: Invalid structure, no structure could be obtained")
        sys.exit(2)

    print("\n PyChemia ABINIT Job")
    print(" =======================\n")
    print(" MPI number of procs : ", nparal)
    print(" ABINIT binary       : ", binary)
    print(" ABINIT input_file   : ", inpu_file)
    print(" ABINIT pseudo       : ", pseudo)
    print(" Structure           :\n")
    print(structure)

    wf = open(output_file, 'w')
    data = {'input': {'binary': binary, 'nparal': nparal, 'structure': structure.to_dict}, 'complete': False, }
    json.dump(data, wf, sort_keys=True)
    wf.close()

    cleaner()

    aj = pychemia.code.abinit.AbinitJob()
    aj.initialize(workdir=workdir, input_file='abinit.in',
                  psp_kind=pseudo.split('_')[0],
                  psp_exchange=pseudo.split('_')[1])
    aj.set_inputs()

    if os.path.exists('abinit.out'):
        os.remove('abinit.out')

    aj.run(use_mpi=True, omp_max_threads=2, mpi_num_procs=nparal)
    time.sleep(5)
    ao = pychemia.code.abinit.AbinitOutput('abinit.out')
    old_niter = 0
    while aj.runner.returncode is None:
        aj.runner.poll()
        ao.reload()
        energetics = ao.get_energetics()
        if energetics is not None:
            if len(energetics['iter']) > old_niter:
                print("ITER %4d  ETOT= %15.8f deltaE(h)= %9.2E residm= %9.2E nres2= %9.2E" % (energetics['iter'][-1],
                                                                                              energetics['etot'][-1],
                                                                                              energetics['deltaEh'][-1],
                                                                                              energetics['residm'][-1],
                                                                                              energetics['nres2'][-1]))
                old_niter = len(energetics['iter'])
        time.sleep(60)

    ao.reload()
    energetics = ao.get_energetics()
    occupation_matrix = ao.get_occupation_matrix()
    output = {'energetics': energetics}
    if occupation_matrix is not None:
        output['occupation_matrix'] = occupation_matrix
    if os.path.isfile('abinit-o_OUT.nc'):
        nco = pychemia.code.abinit.netcdf2dict('abinit-o_OUT.nc')
        output['variables'] = nco
    data['output'] = output

    data['complete'] = True

    wf = open(output_file, 'w')
    json.dump(data, wf, sort_keys=True)
    wf.close()

    cleaner()


if __name__ == "__main__":
    main(sys.argv)



class StructureDistances:

    def __init__(self, pcdb):
        self.pcdb = pcdb

    def get_distance(self, ids_pair):
        return self.pcdb.db.distances.find_one({'pair': ids_pair})

    def set_distance(self, ids_pair, distance):
        self.pcdb.db.distances.insert_one({'pair': ids_pair, 'distance': distance})


class FingerPrints:

    def __init__(self, pcdb):
        self.pcdb = pcdb

    def set_fingerprint(self, fingerprint):
        return self.pcdb.db.fingerprints.insert_one(fingerprint)

    def get_fingerprint(self, entry_id):
        return self.pcdb.db.fingerprints.find_one({'_id': entry_id})

    def update(self, entry_id, fingerprint):
        self.pcdb.db.fingerprints.update({'_id': entry_id}, fingerprint)



import os
import time
import json
import numpy as np
from ..dftb import DFTBplus, read_detailed_out
from pychemia.crystal import KPoints
from pychemia import pcm_log, Structure


class KPointConvergence:
    def __init__(self, structure, workdir='.', slater_path='.', waiting=False, energy_tolerance=1E-3,
                 output_file='results.json'):

        self.structure = structure
        self.workdir = workdir
        self.slater_path = slater_path
        self.waiting = waiting
        self.energy_tolerance = energy_tolerance
        if isinstance(slater_path, str):
            self.slater_path = [slater_path]
        self.results = []
        self.output_file = output_file

        dftb = DFTBplus(workdir=self.workdir)
        kpoints = KPoints.optimized_grid(self.structure.lattice, kp_density=10000, force_odd=True)
        dftb.initialize(structure=self.structure, kpoints=kpoints)
        ans = dftb.set_slater_koster(search_paths=self.slater_path)
        if not ans:
            print('Slater-Koster files not complete')

    def run(self):

        n = 10
        dftb = DFTBplus(workdir=self.workdir)
        kpoints = KPoints.optimized_grid(self.structure.lattice, kp_density=10000, force_odd=True)
        dftb.initialize(structure=self.structure, kpoints=kpoints)
        ans = dftb.set_slater_koster(search_paths=self.slater_path)
        if not ans:
            print('Slater-Koster files not complete')
            return

        grid = None
        energies = []

        while True:
            density = n ** 3
            kpoints = KPoints.optimized_grid(self.structure.lattice, kp_density=density, force_odd=True)
            if np.sum(grid) != np.sum(kpoints.grid):
                pcm_log.debug('Trial density: %d  Grid: %s' % (density, kpoints.grid))
                grid = list(kpoints.grid)
                dftb.kpoints = kpoints

                dftb.basic_input()
                dftb.hamiltonian['MaxSCCIterations'] = 50
                if os.path.isfile('charges.bin'):
                    dftb.hamiltonian['ReadInitialCharges'] = True
                dftb.hamiltonian['Mixer'] = {'name': 'DIIS'}
                dftb.set_static()
                dftb.set_inputs()
                dftb.run()
                if self.waiting:
                    dftb.runner.wait()
                while True:
                    if dftb.runner is not None and dftb.runner.poll() is not None:
                        pcm_log.info('Execution completed. Return code %d' % dftb.runner.returncode)
                        filename = dftb.workdir + os.sep + 'detailed.out'
                        if os.path.exists(filename):
                            ret = read_detailed_out(filename)
                            line = 'KPoint_grid= %15s  iSCC= %4d  Total_energy= %10.4f  SCC_error= %9.3E'
                            print(line % (grid, ret['SCC']['iSCC'], ret['total_energy'], ret['SCC']['SCC_error']))
                        else:
                            print('detailed.out could not be found, exiting...')
                            return
                        n += 2
                        energies.append(ret['total_energy'])
                        break
                    time.sleep(10)

                self.results.append({'kp_grid': grid,
                                     'iSCC': ret['SCC']['iSCC'],
                                     'Total_energy': ret['total_energy'],
                                     'SCC_error': ret['SCC']['SCC_error']})
            else:
                n += 2

            if len(energies) > 2 and abs(max(energies[-3:]) - min(energies[-3:])) < self.energy_tolerance:
                break

    def save_json(self):

        wf = open(self.output_file, 'w')
        json.dump(self.results, wf, sort_keys=True, separators=(',\n', ': '))
        wf.close()


def kpoint_convergence():
    st = Structure.load_json('structure.json')
    job = KPointConvergence(st)
    job.run()
    job.save_json()



"""
pychemia.core is the central subpackage in pychemia. It offers three important classes that are widely
used by all other modules.

The class 'Composition' stores and manipulates chemical compositions.

The class 'Structure' is used to store and manipulated chemical structures, ie, a composition with associated geometry
and in the case of crystals, a lattice.

The class 'Element' allow the retrieval of chemical and physical information about chemical species.
"""

from .composition import Composition
from .structure import Structure
from .element import Element

# __all__ = filter(lambda s: not s.startswith('_'), dir())



"""
Routines related to Data mining
"""
from pychemia import HAS_NETWORKX, HAS_PYMONGO

if HAS_NETWORKX and HAS_PYMONGO:
    from .network import NetworkAnalysis

# __all__ = filter(lambda s: not s.startswith('_'), dir())



import os
from .input import VaspInput


__author__ = "Guillermo Avendano-Franco"
__copyright__ = "Copyright 2016"
__version__ = "0.1"
__maintainer__ = "Guillermo Avendano-Franco"
__email__ = "gtux.gaf@gmail.com"
__status__ = "Development"
__date__ = "May 13, 2016"


def read_incar(filename='INCAR'):
    """
    Load the file INCAR in the directory 'path' or
    read directly the file 'path' and return an object
    'inputvars' for pychemia

    :param filename: (str) Filename of a INCAR file format
    :return:
    """
    if os.path.isfile(filename):
        filename = filename
    elif os.path.isdir(filename) and os.path.isfile(filename + '/INCAR'):
        filename += '/INCAR'
    else:
        raise ValueError('[ERROR] INCAR path not found: %s' % filename)

    iv = VaspInput(filename=filename)
    return iv


def write_incar(iv, filepath='INCAR'):
    """
    Takes an object inputvars from pychemia and
    save the file INCAR in the directory 'path' or
    save the file 'path' as a VASP INCAR file

    :param iv: (VaspInput) VASP Input variables
    :param filepath: (str) File path to write the INCAR file
    """

    if os.path.isdir(filepath):
        filename = filepath + '/INCAR'
    else:
        filename = filepath

    iv.write(filename)




import numpy as np
import itertools
from tvtk.api import tvtk
from pychemia import pcm_log
from mayavi import mlab


class LatticePlot:
    def __init__(self, lattice):
        self.lattice = lattice

    def plot(self, points=None):

        tube_rad = max(self.lattice.lengths) / 100.0

        frame, line1, line2, line3 = self.lattice.get_path()
        for i, j, k in [[frame[:, 0], frame[:, 1], frame[:, 2]],
                        [line1[:, 0], line1[:, 1], line1[:, 2]],
                        [line2[:, 0], line2[:, 1], line2[:, 2]],
                        [line3[:, 0], line3[:, 1], line3[:, 2]]]:
            mlab.plot3d(i, j, k, tube_radius=tube_rad, color=(1, 1, 1), tube_sides=24, transparent=True, opacity=0.5)

        if points is not None:
            ip = np.array(points)
            mlab.points3d(ip[:, 0], ip[:, 1], ip[:, 2], tube_rad * np.ones(len(ip)), scale_factor=1)

        return mlab.pipeline

    def plot_wigner_seitz(self, scale=1):

        ws = np.array(self.lattice.get_wigner_seitz())
        points = np.array([])
        for i in ws:
            for j in i:
                points = np.concatenate((points, j))
        points = scale * (points.reshape(-1, 3))

        index = 0
        # triangles = index + np.array(list(itertools.combinations(range(len(ws[0])), 3)))
        # scalars = _np.ones(len(ws[0]))
        # for i in ws[1:]:
        #     index += len(i)
        #     triangles = np.concatenate((triangles, index + _np.array(list(itertools.combinations(range(len(i)), 3)))))
        #     scalars = np.concatenate((scalars, _np.random.random() * _np.ones(len(i))))
        # scalars = _np.ones(len(ws[0]))
        scalars = None
        triangles = None
        for i in ws:
            pcm_log.debug(i)
            iscalars = np.ones(len(i))
            itriangles = index + np.array(list(itertools.combinations(range(len(i)), 3)))
            pcm_log.debug(iscalars)
            pcm_log.debug(itriangles)
            if triangles is None:
                triangles = itriangles
            else:
                triangles = np.concatenate((triangles, itriangles))
            if scalars is None:
                scalars = iscalars
            else:
                scalars = np.concatenate((scalars, iscalars))
            index += len(i)

        # print triangles
        # print scalars
        # The TVTK dataset.
        mesh = tvtk.PolyData(points=points, polys=triangles)
        mesh.point_data.scalars = scalars
        mesh.point_data.scalars.name = 'scalars'

        pipeline = self.plot()
        pipeline.surface(mesh, color=(0.9, 0.1, 0.1), opacity=0.2)
        return pipeline



import json
import argparse
import pychemia
import logging
import numpy as np

if __name__ == '__main__':

    logging.basicConfig(level=logging.DEBUG)
    logger = logging.getLogger('pychemia')
    logger.addHandler(logging.NullHandler())
    logger.setLevel(logging.DEBUG)

    description = """Collect structures from several databases for plotting convex hulls"""

    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('-t', '--host',
                        default='localhost', metavar='server', type=str,
                        help='Hostname or address (default: localhost)')
    parser.add_argument('-o', '--port',
                        default=27017, metavar='port', type=int,
                        help='MongoDB port (default: 27017)')
    parser.add_argument('-u', '--user',
                        default=None, metavar='username', type=str,
                        help='Username (default: None)')
    parser.add_argument('-p', '--passwd',
                        default=None, metavar='password', type=str,
                        help='Password (default: None)')
    parser.add_argument('-d', '--dbname',
                        default=None, metavar='dbname', type=str, nargs='+',
                        help='PyChemia Database name (default: None)')
    parser.add_argument('-l', '--left',
                        default=None, metavar='specie', type=str,
                        help='Specie that will be on the left side')
    parser.add_argument('-r', '--right',
                        default=None, metavar='specie', type=str,
                        help='Specie that will be on the right side')
    parser.add_argument('--replicaset',
                        default=None, metavar='name', type=str,
                        help='ReplicaSet  (default: None)')
    parser.add_argument('--ssl', action='store_true',
                        help='Use SSL to connect to MongoDB  (default: No)')

    args = parser.parse_args()
    if args.dbname is None:
        parser.print_help()
        exit(1)

    print(args)
    db_settings = {'host': args.host, 'port': args.port, 'ssl': args.ssl, 'replicaset': args.replicaset}
    if args.user is not None:
        if args.passwd is None:
            raise ValueError('Password is mandatory if user is entered')
        db_settings['user'] = args.user
        db_settings['passwd'] = args.passwd

    convex_params = {'left': args.left, 'right': args.right}
    specie_right = args.right
    specie_left = args.left

    ret = []

    for idb in args.dbname:
        db_settings['name'] = idb
        pcdb = pychemia.db.get_database(db_settings)

        for entry in pcdb.entries.find():
            entry_id = entry['_id']
            st = pcdb.get_structure(entry_id)
            formula = st.formula
            symmetry = pychemia.crystal.CrystalSymmetry(st)
            space_group = symmetry.number(symprec=1e-1)

            if specie_left not in st.composition:
                ratio = 1.0
            elif specie_right not in st.composition:
                ratio = 0.0
            else:
                ratio = float(st.composition[specie_right]) / st.natom
                print("ratio: %f - right=%d natom=%d %s" % (ratio,
                                                            float(st.composition[specie_right]),
                                                            st.natom,
                                                            st.formula))

            if 'energy_pa' in entry['properties']:

                energy_pa = entry['properties']['energy_pa']

                ret.append({'formula': formula,
                            'spcgrp': space_group,
                            'energy_pa': energy_pa,
                            'ratio': ratio,
                            'natom': st.natom,
                            'density': st.density})
            else:
                print('ERROR: DB=%s ENTRY=%s No value of energy' % (idb, entry_id))

    sorter = []
    for i in ret:
        sorter.append(1E5 * i['ratio'] + i['energy_pa'])

    npsorter = np.array(sorter)
    srt = npsorter.argsort()

    newdata = []
    for i in srt:
        newdata.append(ret[i])

    wf = open('convex.json', 'w')
    json.dump(newdata, wf, sort_keys=True, indent=4, separators=(',', ': '))
    wf.close()



"""
Created on April 25 2020 
@author: Pedram Tavadze
"""
import os
import numpy as np
from .xml_output import parse_vasprun
from .incar import VaspInput
from ..codes import CodeOutput
from ...core import Structure 
from ...visual import DensityOfStates
from ...crystal.kpoints import KPoints



class VaspXML(CodeOutput):
    
    def __init__(self, filename='vasprun.xml'):
        
        CodeOutput.__init__(self)
        if not os.path.isfile(filename):
            raise ValueError('File not found ' + filename)
        else:
            self.filename = filename
        
        self.spins_dict = {'spin 1':'Spin-up','spin 2':'Spin-down'}
        # self.positions = None
        # self.stress = None
        self.bands = None
        #self.array_sizes = {}
        self.data = self.read()


    def read(self):
        return parse_vasprun(self.filename)
    
    

               
    def _get_dos_total(self):

        spins = list(self.data['general']['dos']['total']['array']['data'].keys())
        energies = np.array(self.data['general']['dos']['total']['array']['data'][spins[0]])[:,0]
        dos_total = {'energies':energies}
        for ispin in spins:
            dos_total[self.spins_dict[ispin]] = np.array(self.data['general']['dos']['total']['array']['data'][ispin])[:,1]
        
        return dos_total,list(dos_total.keys())


    def _get_dos_projected(self,atoms=[]):
        
        if len(atoms) == 0:
            atoms = np.arange(self.initial_structure.natom)
       
        if 'partial' in self.data['general']['dos']:
            dos_projected = {}
            ion_list = ["ion %s"%str(x+1) for x in atoms] # using this name as vasrun.xml uses ion #
            for i in range(len(ion_list)):
                iatom = ion_list[i]
                name = self.initial_structure.symbols[atoms[i]]+str(atoms[i])
                spins = list(self.data['general']['dos']['partial']['array']['data'][iatom].keys())
                energies = np.array(self.data['general']['dos']['partial']['array']['data'][iatom][spins[0]][spins[0]])[:,0]
                dos_projected[name] ={'energies':energies}
                for ispin in spins:
                    dos_projected[name][self.spins_dict[ispin]] =np.array(self.data['general']['dos']['partial']['array']['data'][iatom][ispin][ispin])[:,1:]
            return dos_projected,self.data['general']['dos']['partial']['array']['info']
        else:
            print("This calculation does not include partial density of states")
            return None,None

    
    @property
    def dos_to_dict(self):
        """
        Returns the complete density (total,projected) of states as a python dictionary        
        """
        return {'total':self._get_dos_total(),'projected':self._get_dos_projected()}
    
    @property
    def dos_total(self):
        """
        Returns the total density of states as a pychemia.visual.DensityOfSates object
        """
        dos_total,labels = self._get_dos_total()
        dos_total['energies']-=self.fermi
        return DensityOfStates(np.array([dos_total[x] for x in dos_total]).T, 
                                         title='Total Density Of States',labels=[x.capitalize() for x in labels])

    @property
    def dos_projected(self): 
        """
        Returns the a list of projected density of states as a pychemia.visual.DensityOfSates object
        each element refers to each atom
        """
        ret = []
        atoms = np.arange(self.initial_structure.natom,dtype=int)
        dos_projected,info = self._get_dos_projected(atoms=atoms)
        if dos_projected == None:
            return None
        ndos = len(dos_projected[list(dos_projected.keys())[0]]['energies'])
        norbital = len(info)-1
        nspin = len(dos_projected[list(dos_projected.keys())[0]].keys()) -1
        info[0] = info[0].capitalize()
        labels = []
        labels.append(info[0])
        if nspin >1 :
            for il in info[1:]:
                labels.append(il+'-Up')
            for il in info[1:]:
                labels.append(il+'-Down')
        else : 
            labels = info
        for iatom in dos_projected:
            table = np.zeros(shape = (ndos,norbital*nspin+1))
            table[:,0] = dos_projected[iatom]['energies'] - self.fermi
            start = 1
            for key in dos_projected[iatom]:
                if key == 'energies':
                    continue
                end = start + norbital
                table[:,start:end] = dos_projected[iatom][key]
                start = end 
            temp_dos = DensityOfStates(table,title='Projected Density Of States %s'%iatom,labels=labels)
            ret.append(temp_dos)
        return ret



    def dos_parametric(self,atoms=None,orbitals=None,spin=None,title=None):
        """
        This function sums over the list of atoms and orbitals given 
        for example dos_paramateric(atoms=[0,1,2],orbitals=[1,2,3],spin=[0,1])
        will sum all the projections of atoms 0,1,2 and all the orbitals of 1,2,3 (px,py,pz)
        and return separatly for the 2 spins as a DensityOfStates object from pychemia.visual.DensityofStates
        
        :param atoms: list of atom index needed to be sumed over. count from zero with the same 
                      order as POSCAR
        
        :param orbitals: list of orbitals needed to be sumed over 
        |  s  ||  py ||  pz ||  px || dxy || dyz || dz2 || dxz ||x2-y2||
        |  0  ||  1  ||  2  ||  3  ||  4  ||  5  ||  6  ||  7  ||  8  ||
        
        :param spin: which spins to be included. count from 0
                     There are no sum over spins
        
        """
        projected = self.dos_projected
        if atoms == None :
            atoms = np.arange(self.initial_structure.natom,dtype=int)
        if spin == None :
            spin = [0,1]
        if orbitals == None :
            orbitals = np.arange((len(projected[0].labels)-1)//2,dtype=int)
        if title == None:
            title = 'Sum'
        orbitals = np.array(orbitals)
        if len(spin) ==2:
            labels = ['Energy','Spin-Up','Spin-Down']
            new_orbitals = []
            for ispin in spin :
                new_orbitals.append(list(orbitals+ispin*(len(projected[0].labels)-1)//2))
            orbitals = new_orbitals
        else : 
            if spin[0] == 0:
                labels = ['Energy','Spin-Up']
            elif spin[0] == 1:
                labels = ['Energy','Spin-Down']
            
        
        
        ret = np.zeros(shape=(len(projected[0].energies),len(spin)+1))
        ret[:,0] = projected[0].energies
        
        for iatom in atoms :
            if len(spin) == 2 :
                ret[:,1:]+=self.dos_projected[iatom].values[:,orbitals].sum(axis=2)
            elif len(spin) == 1 :
                ret[:,1]+=self.dos_projected[iatom].values[:,orbitals].sum(axis=1)
                
        return DensityOfStates(table=ret,title=title,labels=labels)
        
    
    def get_band_projection(self):
        return 

    

    @property 
    def kpoints(self):
        """
        Returns the kpoints used in the calculation in form of a pychemia.core.KPoints object
        """
        
        if self.data['kpoints_info']['mode'] == 'listgenerated':
            kpoints = KPoints(kmode='path',kvertices=self.data['kpoints_info']['kpoint_vertices'])
        else :
            kpoints = KPoints(kmode=self.data['kpoints_info']['mode'].lower(),
                               grid=self.data['kpoints_info']['kgrid'],
                               shifts=self.data['kpoints_info']['user_shift'])
        return kpoints
    
    @property
    def kpoints_list(self):
        """
        Returns the list of kpoints and weights used in the calculation in form of a pychemia.core.KPoints object
        """
        return KPoints(kmode='reduced',kpoints_list=self.data['kpoints']['kpoints_list'], 
                       weights=self.data['kpoints']['k_weights'])  
    
    @property 
    def incar(self):
        """
        Returns the incar parameters used in the calculation as pychemia.code.vasp.VaspIncar object
        """
        return VaspInput(variables=self.data['incar'])
        
    @property
    def final_data(self):
        """
        Returns the final free energy, energy_wo_entropy and energy_sigma>0 as a python dictionary
        """
        return {'energy':{'free_energy':self.iteration_data[-1]['energy']['e_fr_energy'],
                          'energy_without_entropy':self.iteration_data[-1]['energy']['e_wo_entrp'],
                          'energy(sigma->0)':self.iteration_data[-1]['energy']['e_0_energy']}}

    @property 
    def vasp_parameters(self):
        """
        Returns all of the parameters vasp has used in this calculation
        """
        return self.data['vasp_params']

    @property 
    def potcar_info(self):
        """
        Returns the information about pseudopotentials(POTCAR) used in this calculation
        """
        return self.data['atom_info']['atom_types']

    @property 
    def fermi(self):
        """
        Returns the fermi energy
        """
        return self.data['general']['dos']['efermi']

    @property
    def species(self):
        """
        Returns the species in POSCAR
        """
        return self.initial_structure.species

    @property 
    def structures(self):
        """
        Returns a list of pychemia.core.Structure representing all the ionic step structures
        """
        symbols = [x.strip() for x in self.data['atom_info']['symbols']]
        structures = []
        for ist in self.data['structures']:
            structures.append(Structure(symbols=symbols,reduced=ist['reduced'],cell=ist['cell']))
        return structures
    
    
    @property 
    def forces(self):
        """
        Returns all the forces in ionic steps
        """
        return self.data['forces']
    
    
    @property
    def initial_structure(self):
        """
        Returns the initial Structure as a pychemia structure
        """
        return self.structures[0]

    @property 
    def final_structure(self):
        """
        Returns the final Structure as a pychemia structure
        """
        
        return self.structures[-1]

    @property 
    def iteration_data(self):
        """
        Returns a list of information in each electronic and ionic step of calculation
        """
        return self.data['calculation']
    
    @property 
    def energies(self):
        """
        Returns a list of energies in each electronic and ionic step [ionic step,electronic step, energy]
        """
        scf_step = 0
        ion_step = 0
        double_counter = 1
        energies = []
        for calc in self.data['calculation']:
            if 'ewald' in calc['energy']:
                if double_counter == 0 :
                    double_counter+=1
                    scf_step +=1
                elif double_counter == 1:
                    double_counter = 0
                    ion_step += 1
                    scf_step = 1  
            else :
                scf_step += 1
            energies.append([ion_step,scf_step,calc['energy']['e_0_energy']])
        return energies

    @property
    def last_energy(self):
        """
        Returns the last calculated energy of the system
        """
        return self.energies[-1][-1]

    @property 
    def energy(self):
        """
        Returns the last calculated energy of the system
        """
        return self.last_energy

    @property
    def convergence_electronic(self):
        """
        Returns a boolian representing if the last electronic self-consistent 
        calculation converged
        """
        ediff = self.vasp_parameters['electronic']['EDIFF']
        last_dE = abs(self.energies[-1][-1]-self.energies[-2][-1])
        if last_dE < ediff :
            return True
        else :
            return False
        
    @property
    def convergence_ionic(self):
        """
        Returns a boolian representing if the ionic part of the  
        calculation converged
        """
        energies = np.array(self.energies)
        nsteps = len(np.unique(np.array(self.energies)[:,0]))
        if nsteps == 1:
            print('This calculation does not have ionic steps')
            return True
        else :
            ediffg = self.vasp_parameters['ionic']['EDIFFG']
            if ediffg < 0 :
                last_forces_abs = np.abs(np.array(self.forces[-1]))
                return not(np.any(last_forces_abs > abs(ediffg)))
            else :
                last_ionic_energy = energies[(energies[:,0] == nsteps)][-1][-1]
                penultimate_ionic_energy = energies[(energies[:,0] == (nsteps-1))][-1][-1]
                last_dE = abs(last_ionic_energy-penultimate_ionic_energy)
                if last_dE < ediffg:
                    return True
        return False
            
    @property
    def convergence(self):
        """
        Returns a boolian representing if the the electronic self-consistent 
        and ionic calculation converged
        """
        return (self.convergence_electronic and self.convergence_ionic)
    
    
    @property
    def is_finished(self):
        """
        Always returns True, need to fix this according to reading the xml as if the calc is 
        not finished we will have errors in xml parser
        """
        # if vasprun.xml is read the calculation is finished
        return True


import json
import logging.handlers
import os
import shutil
import time

import numpy as np

from pychemia import pcm_log
from pychemia.crystal import KPoints
from pychemia.utils.serializer import generic_serializer
from ..outcar import VaspOutput, read_vasp_stdout
from ..poscar import read_poscar
from ..vasp import VaspJob, VaspAnalyser, VaspOutput
from ..input import VaspInput
from ...relaxator import Relaxator
from ...tasks import Task


__author__ = 'Guillermo Avendano-Franco'


class IonRelaxation2(Relaxator, Task):
    def __init__(self, structure, workdir='.', target_forces=1E-3, waiting=False, executable='vasp',
                 encut=1.3, kp_grid=None, kp_density=1E4, relax_cell=True):

        Relaxator.__init__(self, target_forces)
        self.target_forces = target_forces
        self.waiting = waiting
        self.vaspjob = VaspJob(workdir=workdir, executable=executable)
        self.relaxed = False
        if kp_grid is not None:
            self.kpoints = KPoints(kmode='gamma', grid=kp_grid)
        else:
            self.kpoints = KPoints.optimized_grid(structure.lattice, kp_density=kp_density)
        self.vaspjob.initialize(structure=structure, kpoints=self.kpoints)
        self.encut = encut
        self.relax_cell = relax_cell
        task_params = {'target_forces': self.target_forces, 'encut': self.encut, 'relax_cell': self.relax_cell}
        Task.__init__(self, structure=structure, task_params=task_params, workdir=workdir, executable=executable)
        self.stage = 1

    def create_dirs(self, clean=False):
        if not os.path.isdir(self.workdir):
            os.makedirs(self.workdir)
        elif clean:
            for i in os.listdir(self.workdir):
                shutil.rmtree(self.workdir + os.sep + i)

    def update(self):
        """
        This routine determines how to proceed with the relaxation
        for one specific work directory

        :return:
        """
        vj = self.vaspjob

        if self.stage == 1:

            vj.input_variables.variables['PREC'] = 'LOW'
            vj.input_variables.variables['EDIFF'] = 1e-2
            vj.input_variables.variables['EDIFFG'] = 1e-1
            vj.input_variables.set_encut(1.0, POTCAR=self.workdir + os.sep + 'POTCAR')
            vj.input_variables.variables['NSW'] = 65
            vj.input_variables.variables['ISIF'] = 4
            vj.input_variables.variables['IBRION'] = 2
            vj.input_variables.variables['POTIM'] = 0.02
            vj.input_variables.variables['ISMEAR'] = 1
            vj.input_variables.variables['SIGMA'] = 0.10

        elif self.stage == 2:
            vj.input_variables.variables['PREC'] = 'NORMAL'
            vj.input_variables.variables['EDIFF'] = 1e-3
            vj.input_variables.variables['EDIFFG'] = 1e-2
            vj.input_variables.set_encut(1.1, POTCAR=self.workdir + os.sep + 'POTCAR')
            vj.input_variables.variables['NSW'] = 55
            vj.input_variables.variables['ISIF'] = 4
            vj.input_variables.variables['IBRION'] = 1
            vj.input_variables.variables['POTIM'] = 0.30
            vj.input_variables.variables['ISMEAR'] = 1
            vj.input_variables.variables['SIGMA'] = 0.08

        elif self.stage == 3:
            vj.input_variables.variables['PREC'] = 'NORMAL'
            vj.input_variables.variables['EDIFF'] = 1e-3
            vj.input_variables.variables['EDIFFG'] = 1e-2
            vj.input_variables.set_encut(1.2, POTCAR=self.workdir + os.sep + 'POTCAR')  # Originally     ENCUT=520.0
            vj.input_variables.variables['NSW'] = 65
            vj.input_variables.variables['ISIF'] = 3
            vj.input_variables.variables['IBRION'] = 2
            vj.input_variables.variables['POTIM'] = 0.02
            vj.input_variables.variables['ISMEAR'] = 1
            vj.input_variables.variables['SIGMA'] = 0.07

        elif self.stage == 4:
            vj.input_variables.variables['PREC'] = 'NORMAL'
            vj.input_variables.variables['EDIFF'] = 1e-4
            vj.input_variables.variables['EDIFFG'] = 1e-3
            vj.input_variables.set_encut(1.3, POTCAR=self.workdir + os.sep + 'POTCAR')  # Originally     ENCUT=600.0
            vj.input_variables.variables['NSW'] = 55
            vj.input_variables.variables['ISIF'] = 3
            vj.input_variables.variables['IBRION'] = 1
            vj.input_variables.variables['POTIM'] = 0.30
            vj.input_variables.variables['ISMEAR'] = 1
            vj.input_variables.variables['SIGMA'] = 0.06

        elif self.stage == 5:
            vj.input_variables.variables['PREC'] = 'NORMAL'
            vj.input_variables.variables['EDIFF'] = 1e-4
            vj.input_variables.variables['EDIFFG'] = 1e-3
            vj.input_variables.set_encut(1.4, POTCAR=self.workdir + os.sep + 'POTCAR')  # Originally     ENCUT=600.0
            vj.input_variables.variables['NSW'] = 0
            vj.input_variables.variables['ISIF'] = 2
            vj.input_variables.variables['IBRION'] = 2
            vj.input_variables.variables['POTIM'] = 0.02
            vj.input_variables.variables['ISMEAR'] = 1
            vj.input_variables.variables['SIGMA'] = 0.05

        elif self.stage == 6:
            vj.input_variables.variables['PREC'] = 'NORMAL'
            vj.input_variables.variables['EDIFF'] = 0.1 * self.target_forces
            vj.input_variables.variables['EDIFFG'] = 0.5 * self.target_forces
            vj.input_variables.set_encut(1.5, POTCAR=self.workdir + os.sep + 'POTCAR')  # Originally     ENCUT=600.0
            vj.input_variables.variables['NSW'] = 0
            vj.input_variables.variables['ISIF'] = 2
            vj.input_variables.variables['IBRION'] = 2
            vj.input_variables.variables['POTIM'] = 0.02
            vj.input_variables.variables['ISMEAR'] = 1
            vj.input_variables.variables['SIGMA'] = 0.05

        pcm_log.debug('STAGE: %d Current Values: ISIF: %2d   IBRION: %2d   EDIFF: %7.1E \tEDIFFG: %7.1E' %
                      (self.stage,
                       vj.input_variables.variables['ISIF'],
                       vj.input_variables.variables['IBRION'],
                       vj.input_variables.variables['EDIFF'],
                       vj.input_variables.variables['EDIFFG']))

        vj.input_variables.variables['LWAVE'] = False

        for i in ['POSCAR', 'INCAR', 'OUTCAR', 'vasprun.xml']:
            if not os.path.exists(self.workdir + os.sep + i):
                wf = open(self.workdir + os.sep + i, 'w')
                wf.write('')
                wf.close()
            log = logging.handlers.RotatingFileHandler(self.workdir + os.sep + i, maxBytes=1, backupCount=1000)
            log.doRollover()

        vj.structure = self.get_final_geometry()
        vj.set_inputs()
        vj.save_json(self.workdir + os.sep + 'vaspjob.json')

        self.stage += 1

        return True

    def first_run(self, nparal=4):
        print('FIRST RUN')
        print('=========')
        vj = self.vaspjob
        vj.clean()
        inp = VaspInput()
        inp.set_rough_relaxation()
        vj.set_input_variables(inp)
        vj.input_variables.variables['PREC'] = 'LOW'
        vj.input_variables.variables['EDIFF'] = 1e-2
        vj.input_variables.variables['EDIFFG'] = 1e-1
        vj.input_variables.variables['NSW'] = 65
        vj.input_variables.variables['ISIF'] = 4
        vj.input_variables.variables['IBRION'] = 2
        vj.input_variables.variables['POTIM'] = 0.02
        vj.input_variables.variables['ISMEAR'] = 1
        vj.input_variables.variables['SIGMA'] = 0.10
        vj.input_variables.variables['LWAVE'] = False
        vj.write_potcar()
        vj.input_variables.set_encut(ENCUT=self.encut, POTCAR=self.workdir + os.sep + 'POTCAR')
        vj.input_variables.set_density_for_restart()
        vj.set_kpoints(self.kpoints)
        vj.set_inputs()
        print('Launching VASP using %d processes' % nparal)
        vj.run(num_threads=nparal, mpi_num_procs=nparal, nodefile=None, wait=True, verbose=False)
        if self.waiting:
            vj.runner.wait()

    def cleaner(self):

        files = [x for x in os.listdir(self.workdir) if os.path.isfile(self.workdir + os.sep + x)]
        for i in files:
            if i[:5] in ['INCAR', 'POSCA', 'OUTCA', 'vaspr']:
                os.remove(self.workdir + os.sep + i)

    def run(self, nparal=1):

        self.started = True
        self.cleaner()
        vj = self.vaspjob
        ncalls = 1
        self.first_run(nparal)

        while True:
            if vj.runner is not None and vj.runner.poll() is not None:
                pcm_log.info('Execution completed. Return code %d' % vj.runner.returncode)

                filename = self.workdir + os.sep + 'vasp_stdout.log'
                if os.path.exists(filename):
                    read_vasp_stdout(filename=filename)

                ncalls += 1
                va = VaspAnalyser(self.workdir)
                va.run()

                print('READING FORCES AND STRESS')

                max_force, max_stress = self.get_max_force_stress()
                if max_force is not None and max_stress is not None:
                    pcm_log.debug('Max Force: %9.3E Stress: %9.3E' % (max_force, max_stress))
                    vo = VaspOutput(self.workdir + os.sep + 'OUTCAR')
                    info = vo.relaxation_info()
                    pcm_log.debug('Avg Force: %9.3E Stress: %9.3E %9.3E' % (info['avg_force'],
                                                                            info['avg_stress_diag'],
                                                                            info['avg_stress_non_diag']))
                    if self.stage == 7 and info['avg_force'] < self.target_forces and \
                            info['avg_stress_diag'] < self.target_forces and \
                            info['avg_stress_non_diag'] < self.target_forces:
                        break

                else:
                    print('Failure to get forces and stress')

                print('UPDATING INPUTS FOR NEXT RUN')

                self.update()

                vj.run(num_threads=nparal, mpi_num_procs=nparal, nodefile=None, wait=True, verbose=False)
                if self.waiting:
                    vj.runner.wait()

            else:
                filename = self.workdir + os.sep + 'vasp_stdout.log'
                if os.path.exists(filename):
                    vasp_stdout = read_vasp_stdout(filename=filename)
                    if len(vasp_stdout['iterations']) > 0:
                        pcm_log.debug('[%s] SCF: %s' % (os.path.basename(self.workdir), str(vasp_stdout['iterations'])))

                # if os.path.isfile(self.workdir + os.sep + 'OUTCAR'):
                #     vj.get_outputs()

                time.sleep(30)

        # outcars = sorted([x for x in os.listdir(self.workdir) if x.startswith('OUTCAR')])[::-1]
        # vo = VaspOutput(self.workdir + os.sep + outcars[0])
        # forces = vo.forces
        # stress = vo.stress
        # if len(outcars) > 1:
        #     for i in outcars[1:]:
        #         vo = VaspOutput(self.workdir + os.sep + i)
        #         forces = np.concatenate((forces, vo.forces))
        #         stress = np.concatenate((stress, vo.stress))

#        vj.get_outputs()
        # self.output = {'forces': generic_serializer(forces), 'stress': generic_serializer(stress),
        #                'energy': vj.outcar.energy, 'energies': generic_serializer(vj.outcar.energies)}
#        if vj.outcar.is_finished:
#            self.finished = True

    def get_forces_stress_energy(self):

        filename = self.workdir + os.sep + 'OUTCAR'
        if os.path.isfile(filename):
            vo = VaspOutput(filename)
            if vo.has_forces_stress_energy():
                forces = vo.forces[-1]
                stress = vo.stress[-1]
                total_energy = vo.final_data['energy']['free_energy']
            else:
                print('ERROR: VaspOuput says no forces')
                forces = None
                stress = None
                total_energy = None
        else:
            print('ERROR: No OUTCAR')
            forces = None
            stress = None
            total_energy = None
        return forces, stress, total_energy

    def get_final_geometry(self):

        filename = self.workdir + os.sep + 'CONTCAR'
        structure = None
        if os.path.isfile(filename):
            try:
                structure = read_poscar(filename)
            except ValueError:
                print('Error reading CONTCAR')
        return structure

    def plot(self, filedir=None, file_format='pdf'):
        if filedir is None:
            filedir = self.workdir
        import matplotlib.pyplot as plt
        plt.switch_backend('agg')

        plt.figure(figsize=(8, 6))
        plt.subplots_adjust(left=0.1, bottom=0.08, right=0.95, top=0.95, wspace=None, hspace=None)
        forces = np.array(self.output['forces'])
        maxforce = [np.max(np.apply_along_axis(np.linalg.norm, 1, x)) for x in forces]
        avgforce = [np.mean(np.apply_along_axis(np.linalg.norm, 1, x)) for x in forces]

        if np.max(maxforce) > 0.0 and np.max(avgforce) > 0.0:
            plt.semilogy(maxforce, 'b.-', label='Max force')
            plt.semilogy(avgforce, 'r.-', label='Mean force')
        else:
            plt.plot(maxforce, 'b.-', label='Max force')
            plt.plot(avgforce, 'r.-', label='Mean force')
        plt.xlabel('Ion movement iteration')
        plt.ylabel('Max Force')
        plt.savefig(filedir + os.sep + 'forces.' + file_format)
        plt.clf()

        plt.figure(figsize=(8, 6))
        plt.subplots_adjust(left=0.1, bottom=0.08, right=0.95, top=0.95, wspace=None, hspace=None)
        stress = np.array(self.output['stress'])
        diag_stress = [np.trace(np.abs(x)) for x in stress]
        offdiag_stress = [np.sum(np.abs(np.triu(x, 1).flatten())) for x in stress]
        plt.semilogy(diag_stress, 'b.-', label='diagonal')
        plt.semilogy(offdiag_stress, 'r.-', label='off-diagonal')
        plt.legend()
        plt.xlabel('Ion movement iteration')
        plt.ylabel(r'$\sum |stress|$ (diag, off-diag)')
        plt.savefig(filedir + os.sep + 'stress.' + file_format)

    def report(self, file_format='html'):
        from lxml.builder import ElementMaker, E

        self.plot(filedir=self.report_dir, file_format='jpg')

        element_maker = ElementMaker(namespace=None, nsmap={None: "http://www.w3.org/1999/xhtml"})
        html = element_maker.html(E.head(E.title("VASP Ion Relaxation")),
                                  E.body(E.h1("VASP Ion Relaxation"),
                                         E.h2('Initial Structure'),
                                         E.pre(str(self.structure)),
                                         E.h2('Forces Minimization'),
                                         E.p(E.img(src='forces.jpg', width="800", height="600", alt="Forces")),
                                         E.h2('Stress Minimization'),
                                         E.p(E.img(src='stress.jpg', width="800", height="600", alt="Stress"))
                                         ))

        return self.report_end(html, file_format)

    def load(self, filename=None):
        if filename is None:
            filename = self.workdir + os.sep + 'task.json'
        rf = open(filename)
        data = json.load(rf)
        rf.close()
        self.task_params = data['task_params']
        self.output = data['output']
        self.encut = self.task_params['encut']
        self.target_forces = self.task_params['target_forces']
        self.relax_cell = self.task_params['relax_cell']




"""
PyChemia is a python library for automatize atomistic-level calculations. The library provides classes and routines to
manipulate the geometry of atomic structures, with emphasis on crystals.
It has connectors to manipulate the input for several DFT codes and interpret the output from those codes too.
PyChemia allows the store of structures and properties in a Mongo database, search for new structures of a given
composition, and perform a structural search using several metaheuristic global search algorithms included on PyChemia.
"""

import sys
import logging
import importlib

from pychemia.version import version as __version__
from pychemia.version import author as __author__
from pychemia.version import copyright as __copyright__
from pychemia.version import email as __email__
from pychemia.version import status as __status__
from pychemia.version import date as __date__

# Mandatory dependencies
########################

# These check have more sense during develompent
# Installing with pip will fulfill these dependencies

spec = importlib.util.find_spec("scipy")
HAS_SCIPY = spec is not None

# Versions 1.8.x or before used to be pyspglib
# Removed on (2019-12-11)
# try:
#     try:
#         import spglib as spg
#     except ImportError:
#         from pyspglib import spglib as spg
#     HAS_SPGLIB = True
# except ImportError:
#     HAS_SPGLIB = False

spec = importlib.util.find_spec("spglib")
HAS_SPGLIB = spec is not None

spec = importlib.util.find_spec("matplotlib")
HAS_MATPLOTLIB = spec is not None

if HAS_MATPLOTLIB and 'matplotlib' not in sys.modules:
    import matplotlib

    matplotlib.use('agg')

spec = importlib.util.find_spec("matplotlib")
HAS_MATPLOTLIB = spec is not None

spec = importlib.util.find_spec("psutil")
HAS_PSUTIL = spec is not None

# Optional Dependencies
#######################

spec = importlib.util.find_spec("mayavi")
HAS_MAYAVI = spec is not None

spec = importlib.util.find_spec("vtk")
HAS_VTK = spec is not None

spec = importlib.util.find_spec("pyhull")
HAS_PYHULL = spec is not None

spec = importlib.util.find_spec("networkx")
HAS_NETWORKX = spec is not None

spec = importlib.util.find_spec("pymongo")
HAS_PYMONGO = spec is not None

if HAS_PYMONGO:
    import pymongo

    if pymongo.version_tuple[0] < 3:
        HAS_PYMONGO = False
    else:
        HAS_PYMONGO = True

spec = importlib.util.find_spec("gridfs")
HAS_GRIDFS = spec is not None

spec = importlib.util.find_spec("ase")
HAS_ASE = spec is not None

spec = importlib.util.find_spec("pymatgen")
HAS_PYMATGEN = spec is not None

pcm_log = logging.getLogger(__name__)
pcm_log.addHandler(logging.NullHandler())

from .core import Structure, Composition, Element
from . import analysis
from . import db
from . import crystal
from . import io
from . import runner
from . import searcher
from . import utils
from . import web
from . import code
from . import population
from .core.from_file import structure_from_file


def info():
    """
    Show basic information about PyChemia, the version, location of the package and registered date of release
    Both mandatory and optional dependecies are shown, including version and location.

    """
    print('PyChemia\n--------\n')
    print('Version: ' + __version__)
    print('Path:    ' + __path__[0])
    print('Date:    ' + __date__)
    print()

    import sys
    print('Python version=' + sys.version + '\n')

    for modui in ['numpy', 'scipy', 'spglib', 'matplotlib', 'pymongo', 'psutil',
                  'nose', 'coverage', 'pyhull', 'pymatgen',
                  'networkx', 'ase', 'mayavi', 'qmpy', ]:
        if modui == 'numpy':
            print("Mandatory dependencies:")
        if modui == 'nose':
            print("\nOptional dependencies:")
        try:
            mm = __import__(modui)
            print('%10s %10s   %s' % (modui, mm.__version__, mm.__path__[0]))
        except ImportError:
            print('%10s %10s Not Found' % (modui, ''))

    try:
        import vtk
        print('%10s %10s   %s' % ('vtk', vtk.VTK_VERSION, vtk.__path__[0]))
    except ImportError:
        print('%10s %10s Not Found' % ('vtk', ''))



import re
import os
import numpy as np
from pychemia.utils.serializer import generic_serializer
from pychemia import pcm_log
from ..codes import CodeOutput
from .xml_output import parse_vasprun


class VaspOutput(CodeOutput):

    def __init__(self, filename='OUTCAR'):

        CodeOutput.__init__(self)
        if not os.path.isfile(filename):
            raise ValueError('File not found ' + filename)
        else:
            self.filename = filename
            rf = open(self.filename, 'r')
            self.data = rf.read()
            rf.close()

        self.charge = {}
        self.energies = None
        self.last_energy = None
        self.forces = None
        self.stress = None
        self.fermi = None
        self.bands = None
        self.positions = None
        self.kpoints = None
        self.array_sizes = {}
        self.species = None
        self.final_data = {}
        self.iteration_data = []
        self.outcar_parser()
        self.read()

    def read(self):
        if self.filename[-3:] == 'xml':
            ret = parse_vasprun(self.filename)
        else:
            self.read_outputfile(self.filename)
            self.outcar_parser()
            ret = {'energies': self.energies, 'forces': self.forces, 'bands': self.bands, 'positions': self.positions,
                   'kpoints': self.kpoints, 'stress': self.stress}
        self.output_values = ret

    def is_loaded(self):
        if self.output_values is None:
            return False
        else:
            return True

    def read_outputfile(self, filename=None):
        if filename is not None and os.path.exists(filename):
            rf = open(filename)
        else:
            rf = open(self.filename)
        self.data = rf.read()
        rf.close()

    def reload(self):
        rf = open(self.filename)
        self.data = rf.read()
        rf.close()

    def outcar_parser(self):

        for istr in ['NKPTS', 'NBANDS', 'NEDOS', 'NIONS', 'NGX', 'NGY', 'NGZ', 'NGXF', 'NGYF', 'NGZF', 'ISPIN']:
            redata = re.findall(istr + r'\s*=\s*(\d+)', self.data)
            if len(redata) > 0:
                self.array_sizes[istr] = int(redata[0])
        # pcm_log.info('Array sizes : ' + str(self.array_sizes))

        self.species = re.findall(r'POTCAR\s*:\s*[\w_]+\s*(\w+)', self.data)
        self.species = self.species[:int(len(self.species) / 2)]
        # pcm_log.info('Number of species (= number of POTCARs):' + str(self.species))

        pos_forces = re.findall(r'TOTAL-FORCE \(eV/Angst\)\s*-*\s*([-.\d\s]+)\s+-{2}', self.data)
        pos_forces = np.array([x.split() for x in pos_forces], dtype=float)

        if len(pos_forces) > 0 and len(pos_forces[-1]) % 6 == 0:
            pos_forces.shape = (len(pos_forces), -1, 6)
            forces = pos_forces[:, :, 3:]
            positions = pos_forces[:, :, :3]
            pcm_log.debug('Positions from OUTCAR: %d iterations' % len(positions))
            pcm_log.debug('Forces from OUTCAR: %d iterations' % len(forces))

            self.forces = forces
            self.positions = positions
            self.array_sizes['NIONSTEPS'] = len(self.forces)
            pcm_log.debug('Number of Ionic steps: ' + str(self.array_sizes['NIONSTEPS']))
        else:
            print('Forces and Positions could not be parsed : ', pos_forces.shape)
            print('pos_forces =\n%s ' % pos_forces)

        fermi = re.findall(r'E-fermi\s+:\s+([-.\d]+)', self.data)
        fermi = np.array(fermi, dtype=float)
        # pcm_log.debug('Fermi Level(eV): ' + str(fermi))
        self.fermi = fermi

        # This regex covers the entire information for each electronic iteration.
        # The expression in the middle, catch everything but ('>' greater than)
        # The final part catch the value of the energy
        # It returns a list of tuples in the form [(ionic iter, Energy (elect. Iter), ...]
        energy = re.findall(r'Iteration\s*(\d+)\s*\(\s*(\d+)\)[-+*/=():.\s\d\w]+>0\)\s*=\s*([-.\d]+)', self.data)
        # pcm_log.debug('Energy(eV) [(ionic iter, Energy (elect. Iter)),...]: ' + str(energy))

        self.energies = []
        index = None

        for ienergy in energy:
            self.energies.append([int(ienergy[0]), int(ienergy[1]), float(ienergy[2])])

        ### BAD idea this way
        ###re.findall(r'free\s*energy\s*TOTEN\s*=\s*([-.\d]+)\s*eV', data)
        if not energy:
            energy = re.findall(
                r'FREE ENERGIE OF THE ION-ELECTRON SYSTEM\s*\(eV\)\s*[-]+\s*free\s*energy\s*TOTEN\s*=\s*([-.\d]+)\s*eV',
                self.data)

            counter = 0
            for ienergy in energy:
                self.energies.append([counter, 1, float(ienergy)])
                counter += 1

        level0 = 0
        level1 = 0
        for i in self.energies:
            if i[0] > level0:
                self.last_energy = i[2]
                level0 = i[0]
            elif i[0] == level0 and i[1] > level1:
                self.last_energy = i[2]
                level1 = i[1]

        pattern = r"k-point([\s\d]+):([\d\s.+-]+)\s*band No.\s*band energies\s*occupation\s*\n([\d\w\s.+-]+)\n\n"
        bands = re.findall(pattern, self.data)
        bands_dict = {}
        for iband in bands:
            try:
                ikpt = int(iband[0])
                kpt_pos = [float(x) for x in iband[1].split()]
                kpt_eigenvals = np.array([float(x) for x in iband[2].split()[:3 * self.array_sizes['NBANDS']]])
                kpt_eigenvals.reshape((-1, 3))
                bands_dict[ikpt] = {'position': kpt_pos, 'eigenvalues': kpt_eigenvals}
            except ValueError:
                print('Error parsing bands')
        self.bands = bands_dict

        if False and 'NIONSTEPS' in self.array_sizes:
            if len(bands) != self.array_sizes['NBANDS'] * self.array_sizes['ISPIN'] * self.array_sizes['NKPTS'] \
                    * self.array_sizes['NIONSTEPS']:
                pcm_log.debug('NBANDS: %s != ISPIN: %s x NKPTS: %s x NIONSTEPS: %s' % (self.array_sizes['NBANDS'],
                                                                                       self.array_sizes['ISPIN'],
                                                                                       self.array_sizes['NKPTS'],
                                                                                       self.array_sizes['NIONSTEPS']))
        # pcm_log.info('Bands : ' + str(bands))

        stress = re.findall(r'in\s+kB ([-*.\s\d]+)external', self.data)
        # Converted from kBar to GPa
        if stress:
            stress = 0.1 * np.array([x.split() if '*' not in x else 6 * [float('nan')] for x in stress], dtype=float)
            stress.reshape((-1, 6))
            nionsteps = len(stress)
            self.stress = np.zeros((nionsteps, 3, 3))
            self.stress[:, 0, 0] = stress[:, 0]
            self.stress[:, 1, 1] = stress[:, 1]
            self.stress[:, 2, 2] = stress[:, 2]
            self.stress[:, 0, 1] = stress[:, 3]
            self.stress[:, 1, 2] = stress[:, 4]
            self.stress[:, 0, 2] = stress[:, 5]
            self.stress[:, 1, 0] = stress[:, 3]
            self.stress[:, 2, 1] = stress[:, 4]
            self.stress[:, 2, 0] = stress[:, 5]
            # pcm_log.info('Stress (GPa):\n ' + str(self.stress))

        charge = re.findall(r'total\s*charge\s*#\s*of\s*ion\s*s\s*p\s*d\s*tot\s*-+([-.\s\d]+)\s--', self.data)
        if charge:
            charge = np.array([x.split() for x in charge], dtype=float)
            charge.shape = (len(charge), -1, 5)
            if len(charge) == 2:
                if np.all(charge[0] != charge[1]):
                    pcm_log.error('Bad Charge')
            charge = charge[0, :, 1:]
            self.charge['s'] = list(charge[:, 0])
            self.charge['p'] = list(charge[:, 1])
            self.charge['d'] = list(charge[:, 2])
            self.charge['total'] = list(charge[:, 3])
            # pcm_log.debug('Charge :' + str(self.charge))

        # Processing Final
        self.free_energy()
        self.iterations()

    def free_energy(self):
        subdata = re.findall('FREE ENERGIE OF THE ION-ELECTRON SYSTEM [\w\d\s()\n-=>]*\n\s*\n\n\n-', self.data)
        if len(subdata) == 1:
            data_split = subdata[0].split()
            try:
                float(data_split[12])
                float(data_split[17])
                float(data_split[20])
            except ValueError:
                raise ValueError('Energy values could not be parsed from: %s' % subdata)
            self.final_data['energy'] = {'free_energy': float(data_split[12]),
                                         'energy_without_entropy': float(data_split[17]),
                                         'energy(sigma->0)': float(data_split[20])}

    @property
    def energy(self):
        if len(self.energies) > 0:
            return self.energies[-1][-1]

    def iterations(self):
        # Capture the data for each iteration. The symbol '>' is not included on purpose to close
        # the capture close to "energy(sigma->0)..."
        subdata = re.findall(r'-+ Iteration\s*(\d+)\(\s*(\d+)\)\s*-+\s*([\s\d\w:.,\-=()/+*]*)energy', self.data)
        for i in subdata:
            io_iter = int(i[0]) - 1
            scf_iter = int(i[1]) - 1
            # print io_iter, scf_iter
            while len(self.iteration_data) <= io_iter:
                self.iteration_data.append([])
            while len(self.iteration_data[io_iter]) <= scf_iter:
                self.iteration_data[io_iter].append([])
            self.iteration_data[io_iter][scf_iter] = {}
            self.iteration_data[io_iter][scf_iter]['Timing'] = {}
            for j in ['POTLOK', 'SETDIJ', 'EDDIAG', 'RMM-DIIS', 'ORTHCH', 'DOS', 'CHARGE', 'MIXING', 'LOOP']:
                iter_block = re.findall(j + r':([\d\w:. ]*)\n', i[2])
                if len(iter_block) > 0:
                    iter_block_split = iter_block[0].split()
                    try:
                        float(iter_block_split[2][:-1])
                        float(iter_block_split[5])
                    except ValueError:
                        raise ValueError('Could not retrieve data', iter_block)
                    self.iteration_data[io_iter][scf_iter]['Timing'][j] = {'cpu': float(iter_block_split[2][:-1]),
                                                                           'real': float(iter_block_split[5])}

            self.iteration_data[io_iter][scf_iter]['Free_Energy'] = {}
            for j in ['PSCENC', 'TEWEN', 'DENC', 'EXHF', 'XCENC', 'EENTRO', 'EBANDS', 'EATOM', 'TOTEN']:
                iter_block = re.findall(j + r'\s*=\s*([\d.-]*)[\s\w\d]*\n', i[2])
                if len(iter_block) > 0:
                    self.iteration_data[io_iter][scf_iter]['Free_Energy'][j] = float(iter_block[0])

    def has_forces_stress_energy(self):
        return self.forces is not None and self.stress is not None and self.last_energy is not None

    def __str__(self):
        ret = '\nForces:\n'
        index = 0
        for iforce in self.forces[-1]:
            index += 1
            ret += "%3d %12.6f %12.6f %12.6f\n" % (index, iforce[0], iforce[1], iforce[2])
        ret += '\nStress:\n'
        for j in range(3):
            ret += '    %12.6f %12.6f %12.6f\n' % tuple(self.stress[-1][j])
        ret += '\n'

        ret += 'Free Energy: %12.6f\n' % self.energy
        return ret

    def relaxation_info(self):

        info = {}
        if self.stress is not None:
            info['avg_stress_diag'] = np.average(np.abs(self.stress[-1].diagonal()))
            info['avg_stress_non_diag'] = np.average(np.abs(np.triu(self.stress[-1], 1)))
        if self.forces is not None:
            info['avg_force'] = np.average(np.abs(np.apply_along_axis(np.linalg.norm, 1, self.forces[-1])))
        return info

    @property
    def to_dict(self):
        ret = {}
        for i in ['charge', 'energy', 'forces', 'stress']:
            ret[i] = eval('self.' + i)

        for i in ret:
            if isinstance(ret[i], np.ndarray):
                ret[i] = generic_serializer(ret[i])
        return ret

    def get_magnetization(self):
        ret = {}
        for mag_dir in ['x', 'y', 'z']:
            mag = re.findall(r'magnetization\s*\(%s\)\s*#\s*of\s*ion\s*s\s*p\s*d\s*tot\s*-+([-.\s\d]+)\s--' % mag_dir,
                             self.data)
            if mag:
                mag = np.array([x.split() for x in mag], dtype=float)
                mag.shape = (len(mag), -1, 5)
                if len(mag) == 2:
                    assert np.all(np.array(mag[0] == mag[1]))
                mag = mag[-1, :, 1:]
                ret[mag_dir] = {}
                ret[mag_dir]['s'] = [float(x) for x in mag[:, 0]]
                ret[mag_dir]['p'] = [float(x) for x in mag[:, 1]]
                ret[mag_dir]['d'] = [float(x) for x in mag[:, 2]]
                ret[mag_dir]['total'] = [float(x) for x in mag[:, 3]]
        return ret

    def get_memory_used(self):

        ret = {}
        datablock = re.findall(r"total amount of memory used by VASP on root node([\w\s\d.=:-]*)\n \n {2}", self.data)
        if len(datablock) != 1:
            return None
        else:
            datablock = datablock[0]
        print(datablock)

        for iline in datablock.split('\n'):
            if ':' in iline:
                key_value = iline.split(':')
                try:
                    ret[key_value[0].strip()] = (float(key_value[1].split()[0]), key_value[1].split()[1])
                except ValueError:
                    print('Failed to parse: ', key_value)
        return ret

    def get_general_timing(self):
        ret = {}
        last_lines = self.data.split('\n')[-15:]
        if 'General timing and accounting informations for this job' in last_lines[0]:
            for iline in last_lines[2:]:
                if ':' in iline:
                    key_value = iline.split(':')
                    try:
                        ret[key_value[0].strip()] = float(key_value[1].strip())
                    except ValueError:
                        print('Failed to parse: ', key_value)
        return ret

    @property
    def is_finished(self):
        return len(self.get_general_timing()) > 0



#!D:\bld\pychemia_1604228883768\_h_env\python.exe

import pychemia
import numpy as np
import os
import json
import sys
import getopt


def usage(name):
    print("""
NAME

    %s

DESCRIPTION
    Collects information about all the structures found during a Minimal Hoping Method using the original
    code from Amsler,

OPTIONS

    --help, -h
        Return information on the options and use of this script

    --directory, -d <string>
        Directory where the structure search stored the data

    --left, -l <string>
        Specie that will appear on the left side of the convex hull, its composition ratio will be 0.0

    --right, -r <string>
        Specie that will appear on the right side of the convex hull, its composition ratio will be 1.0
""" % os.path.basename(name))


def main(argv):
    try:
        opts, args = getopt.getopt(argv[1:], "hd:l:r:", ["help", "directory=", "left=", "right="])
    except getopt.GetoptError:
        usage(argv[0])
        sys.exit(2)

    if len(opts) == 0:
        usage(argv[0])
        sys.exit(2)

    # Default Values
    directory = None
    left = None
    right = None

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage(argv[0])
            sys.exit()
        elif opt in ("-d", "--directory"):
            directory = arg
        elif opt in ("-l", "--left"):
            left = arg
        elif opt in ("-r", "--right"):
            right = arg

    if directory is None or left is None or right is None:
        usage(argv[0])
        sys.exit(1)

    if not os.path.isdir(directory):
        print('Not such directory ', directory)
        sys.exit(1)

    basedir = directory
    specie1 = right
    specie2 = left

    dirs1 = sorted([x for x in os.listdir(basedir) if os.path.isdir(basedir + os.sep + x)])
    ret = []
    for idir in dirs1:
        dirs2 = sorted([x for x in os.listdir(basedir + os.sep + idir) if x[:3] == 'xxx'])
        for idir2 in dirs2:
            path = basedir + os.sep + idir + os.sep + idir2

            if not os.path.exists(path + os.sep + 'CONTCAR') or os.path.getsize(path + os.sep + 'CONTCAR') == 0:
                print("Missing CONTCAR on %s/%s" % (idir, idir2))

                if not os.path.exists(path + os.sep + 'POSCAR') or os.path.getsize(path + os.sep + 'POSCAR') == 0:
                    print("Missing POSCAR on %s/%s" % (idir, idir2))
                    continue
                st = pychemia.code.vasp.read_poscar(path + os.sep + 'POSCAR')
            else:
                st = pychemia.code.vasp.read_poscar(path + os.sep + 'CONTCAR')

            formula = st.formula

            symmetry = pychemia.crystal.CrystalSymmetry(st)
            space_group = symmetry.number(symprec=1e-1)

            vo = pychemia.code.vasp.VaspOutput(path + os.sep + 'OUTCAR')
            if not vo.is_finished:
                continue
            energy = vo.last_energy

            # ana=pychemia.analysis.StructureAnalysis(st)
            # b, c, r = ana.bonds_coordination()

            if vo.forces is not None and len(vo.forces) > 0:
                maxforce = np.max(np.apply_along_axis(np.linalg.norm, 1, vo.forces[-1]))
            else:
                maxforce = 100

            if specie2 not in st.composition:
                ratio = 1
            elif specie1 not in st.composition:
                ratio = 0
            else:
                ratio = float(st.composition[specie1]) / (st.composition[specie2] + st.composition[specie1])

            print(" %30s SPCGRP: %4d   ENERGY_PA: %9.3f   MAXFORCE: %9.2E" % ((idir + os.sep + idir2).ljust(30),
                                                                              space_group, energy / st.natom, maxforce))

            ret.append({'formula': formula,
                        'spcgrp': space_group,
                        'energy': energy,
                        'energy_pa': energy / st.natom,
                        'ratio': ratio,
                        'idir': idir,
                        'idir2': idir2,
                        'natom': st.natom,
                        'maxforce': maxforce,
                        'density': st.density})
    # 'bonds/volume': np.sum(c) / st.volume,
    #                    'max_coordination': np.max(c),
    #                    'coordination': c})

    sorter = []
    for i in ret:
        sorter.append(1E5 * i['ratio'] + i['energy_pa'])

    npsorter = np.array(sorter)
    srt = npsorter.argsort()

    newdata = []
    for i in srt:
        newdata.append(ret[i])

    wf = open('results.json', 'w')
    json.dump(newdata, wf, sort_keys=True, indent=4, separators=(',', ': '))
    wf.close()


if __name__ == "__main__":
    main(sys.argv)



#!D:\bld\pychemia_1604228883768\_h_env\python.exe

import os
import argparse
import logging
import pychemia


if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='Orbital DFT+U Searcher Manager')

    parser.add_argument('-host', type=str, help='Hostname or IP of the Mongo Server (default: localhost)',
                        required=False, metavar='hostname')
    parser.add_argument('-dbname', type=str, help='Name of Database', required=True, metavar='name')
    parser.add_argument('-ssl', help='Use SSL (default: False)', action='store_true')
    parser.add_argument('-generation_size', type=int, help='Generation Size (default: 32)', metavar='N', default=32)
    parser.add_argument('-abinit_input', type=str, help='Path to Abinit input file', metavar='path',
                        default='abinit.in')
    parser.add_argument('-new', help='Create new database (default: False)', action='store_true')
    parser.add_argument('-debug', help='Activate debug mode (default: False)', action='store_true')
    parser.add_argument('-clean', help='Clean database before start (default: False)', action='store_true')

    args = parser.parse_args()

    if args.debug:
        loglevel = logging.DEBUG
    else:
        loglevel = logging.INFO

    logging.basicConfig(level=loglevel)
    logger = logging.getLogger('pychemia')
    logger.addHandler(logging.NullHandler())
    logger.setLevel(loglevel)

    if args.host is None:
        host = 'localhost'
    else:
        host = args.host

    print("Host:                  %s" % host)
    print("Database Name:         %s" % args.dbname)
    print("SSL:                   %s" % args.ssl)
    print("Create new database:   %s" % args.new)
    print("Generation Size:       %s" % args.generation_size)
    print("Abinit input:          %s" % args.abinit_input)

    if not os.path.exists(args.abinit_input):
        raise ValueError("ERROR: File %s not found" % args.abinit_input)

    if args.new:
        print("Creating new database")
        admin_name = input('Admin Username:')
        admin_passwd = input('Admin Password:')

    user = input('Username:')
    if user == '':
        user = None
        passwd = None
    else:
        passwd = input('Password:')

    if args.ssl is None:
        ssl = False
    else:
        ssl = args.ssl

    if args.new:
        pcdb = pychemia.db.create_database(name=args.dbname, admin_name=admin_name, admin_passwd=admin_passwd,
                                           user_name=user, user_passwd=passwd, host=host, ssl=args.ssl)
    else:
        dbsettings = {'host': host, 'name': args.dbname, 'user': user, 'passwd': passwd}
        pcdb = pychemia.db.get_database(dbsettings)

    if args.clean:
        pcdb.clean()

    popu = pychemia.population.orbitaldftu.OrbitalDFTU(pcdb, input_path=args.abinit_input)
    searcher = pychemia.searcher.FireFly(popu, generation_size=args.generation_size)
    print("Starting search...")
    searcher.run()



import numpy as np
from . import procar


class BandStructure:
    def __init__(self, structure, filename='PROCAR'):
        self.structure = structure
        rcell = self.structure.lattice.reciprocal().cell
        print(rcell)
        self.procarFile = procar.ProcarParser()
        self.procarFile.readFile(procar=filename, permissive=False, recLattice=rcell)

        self.data = procar.ProcarSelect(self.procarFile, deepCopy=True)

    def simple_bands(self):
        self.data.selectIspin([0])
        self.data.selectAtoms([-1])
        self.data.selectOrbital([-1])

        fermi = 0

        bands = (self.data.bands.transpose() - np.array(fermi)).transpose()

        plot = procar.ProcarPlot(bands, self.data.spd, self.data.kpoints)

        fig, ax = plot.scatterPlot()
        ax.set_ylabel(r"Energy [eV]")

        return fig, ax



import sys
import numpy as np
from abc import ABCMeta, abstractmethod


class OptimizationTestFunction:
    __metaclass__ = ABCMeta
    """
    General class for Test Functions used for optimization
    """

    def __init__(self, mindim=1, maxdim=None, domain=np.array([-1, 1])):
        self.mindim = mindim
        self.maxdim = maxdim
        self.domain = domain

    @staticmethod
    def function(x):
        return np.sum(np.abs(x))

    @abstractmethod
    def minimum(self, ndim):
        pass

    def fminimum(self, ndim):
        x = self.minimum(ndim)
        return self.function(x)

    def get_plot_matrices(self, shape=None):
        if shape is None:
            shape = [200, 200]
        if self.domain.ndim == 1:
            dx = float(self.domain[1] - self.domain[0]) / (shape[0])
            X, Y = np.mgrid[self.domain[0]:self.domain[1]:dx, self.domain[0]:self.domain[1]:dx]
        else:
            dx = float(self.domain[0, 1] - self.domain[0, 0]) / (shape[0])
            dy = float(self.domain[1, 1] - self.domain[1, 0]) / (shape[1])
            X, Y = np.mgrid[self.domain[0, 0]:self.domain[0, 1]:dx, self.domain[1, 0]:self.domain[1, 1]:dy]
        Z = self.function(np.array([X, Y]))
        return X, Y, Z


class Sphere(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=1, maxdim=None, domain=np.array([-5, 5]))

    @staticmethod
    def function(x):
        x = np.array(x)
        return np.sum(x.T * x.T, axis=-1).T

    def minimum(self, ndim):
        return np.zeros(ndim)


class Ackley(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=1, maxdim=None, domain=np.array([-5, 5]))

    @staticmethod
    def function(x):
        n = len(x)
        exp1 = np.exp(-0.2 * np.sqrt(1.0 / n * np.sum(x * x)))
        exp2 = np.exp(1.0 / n * np.sum((np.cos(2 * np.pi * x)).T, axis=-1).T)
        return -20 * exp1 - exp2 + np.e + 20

    def minimum(self, ndim):
        return np.zeros(ndim)


class Rosenbrock(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=1, maxdim=None, domain=np.array([-5, 5]))

    @staticmethod
    def function(x):
        return np.sum((100.0 * (x[1:] - x[:-1] ** 2.0) ** 2.0 + (1 - x[:-1]) ** 2.0).T, axis=-1).T

    def minimum(self, ndim):
        return np.ones(ndim)


class Beale(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-4.5, 4.5]))

    @staticmethod
    def function(x):
        return (1.5 - x[0] + x[0] * x[1]) ** 2 + (2.25 - x[0] + x[0] * x[1] * x[1]) ** 2 + (2.625 - x[0] + x[0] * x[1] *
                                                                                            x[1] * x[1]) ** 2

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([3.0, 0.5])


class GoldsteinPrice(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-2, 2]))

    @staticmethod
    def function(x):
        factor1 = (19 - 14 * x[0] + 3 * x[0] ** 2 - 14 * x[1] + 6 * x[0] * x[1] + 3 * x[1] ** 2)
        factor2 = (18 - 32 * x[0] + 12 * x[0] ** 2 + 48 * x[1] - 36 * x[0] * x[1] + 27 * x[1] ** 2)
        return (1 + ((x[0] + x[1] + 1) ** 2) * factor1) * (30 + ((2 * x[0] - 3 * x[1]) ** 2) * factor2)

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([0.0, -1.0])


class Booth(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-10, 10]))

    @staticmethod
    def function(x):
        return (x[0] + 2 * x[1] - 7) ** 2 + (2 * x[0] + x[1] - 5) ** 2

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([1.0, -3.0])


class BukinN6(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([[-15, 15], [-3, 3]]))

    @staticmethod
    def function(x):
        return 100 * np.sqrt(np.abs(x[1] - 0.01 * x[0] ** 2)) + 0.01 * np.abs(x[0] + 10)

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([10.0, 1.0])


class Matyas(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-10, 10]))

    @staticmethod
    def function(x):
        return 0.26 * (x[0] ** 2 + x[1] ** 2) - 0.48 * x[0] * x[1]

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([1.0, 1.0])


class LeviN13(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-10, 10]))

    @staticmethod
    def function(x):
        term1 = (np.sin(3 * np.pi * x[0])) ** 3
        term2 = (x[0] - 1) ** 2 * (1 + (np.sin(3 * np.pi * x[1])) ** 3)
        term3 = (x[1] - 1) ** 2 * (1 + (np.sin(2 * np.pi * x[1])) ** 2)
        return term1 + term2 + term3

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([1.0, 1.0])


class ThreeHump(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-10, 10]))

    @staticmethod
    def function(x):
        return 2 * x[1] ** 2 - 1.05 * x[0] ** 4 + 1.0 / 6.0 * x[0] ** 6 + x[0] * x[1] + x[1] ** 2

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([0.0, 0.0])


class Easom(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-100, 100]))

    @staticmethod
    def function(x):
        return -np.cos(x[0]) * np.cos(x[1]) * np.exp(-1 * ((x[0] - np.pi) ** 2 + (x[1] - np.pi) ** 2))

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([np.pi, np.pi])


class CrossInTray(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-10, 10]))

    @staticmethod
    def function(x):
        factor1 = np.exp(np.abs(100 - np.sqrt(x[0] ** 2 + x[1] ** 2) / np.pi))
        return -1E-4 * (np.abs(np.sin(x[0]) * np.sin(x[1]) * factor1) + 1) ** 0.1

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([1.34941, 1.34941])


class Eggholder(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-512, 512]))

    @staticmethod
    def function(x):
        return -1.0 * (x[1] + 47) * np.sin(np.sqrt(np.abs(x[1] + x[0] / 2.0 + 47))) - x[0] * np.sin(
            np.sqrt(np.abs(x[0] - x[1] - 47)))

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([512, 404.2319])


class HolderTable(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-10, 10]))

    @staticmethod
    def function(x):
        return -1.0 * np.abs(np.sin(x[0]) * np.cos(x[1]) * np.exp(np.abs(1 - np.sqrt(x[0] ** 2 + x[1] ** 2) / np.pi)))

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([8.05502, 9.664559])


class McCormick(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([[-1.5, 4], [-3, 4]]))

    @staticmethod
    def function(x):
        return np.sin(x[0] + x[1]) + (x[0] - x[1]) ** 2 - 1.5 * x[0] + 2.5 * x[1] + 1

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([8.05502, 9.66459])


class SchafferN2(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-100, 100]))

    @staticmethod
    def function(x):
        return 0.5 + ((np.sin(x[0] ** 2 - x[1] ** 2)) ** 2 - 0.5) / (1 + 1E-3 * (x[0] ** 2 + x[1] ** 2)) ** 2

    def minimum(self, ndim):
        assert ndim == 2
        return np.zeros(2)


class SchafferN4(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-100, 100]))

    @staticmethod
    def function(x):
        return 0.5 + ((np.cos(np.sin(np.abs(x[0] ** 2 - x[1] ** 2)))) ** 2 - 0.5) / (1 + 1E-3 * (
            x[0] ** 2 + x[1] ** 2)) ** 2

    def minimum(self, ndim):
        assert ndim == 2
        return np.array([0, 1.25313])


class StyblinskiTang(OptimizationTestFunction):
    def __init__(self):
        OptimizationTestFunction.__init__(self, mindim=1, maxdim=None, domain=np.array([-5, 5]))

    @staticmethod
    def function(x):
        return np.sum((x ** 4 - 16 * x ** 2 + 5 * x).T, axis=-1).T / 2.0

    def minimum(self, ndim):
        return -2.903534 * np.ones(ndim)


# class Simionescu(OptimizationTestFunction):
#
#     def __init__(self):
#         OptimizationTestFunction.__init__(self, mindim=2, maxdim=2, domain=np.array([-1.25, 1.25]))
#
#     @staticmethod
#     def function(x):
#         rt = 1
#         rs = 0.2
#         n = 8
#         return np.piecewise(x,
#                      [x[0]**2 + x[1]**2 <= (rt + rs*np.cos(n*np.arctan(x[0]/x[1])))**2,
#                       x[0]**2 + x[1]**2 > (rt + rs*np.cos(n*np.arctan(x[0]/x[1])))**2], [0.1*x[0]*x[1], 1])
#
#
#     def minimum(self, ndim):
#         assert ndim == 2
#         return -0.84852813*np.ones(ndim)


def all_tests_functions():
    current_module = sys.modules[__name__]
    f = current_module.__dict__
    return [f[x]() for x in f if hasattr(f[x], '__base__') and f[x].__base__ == OptimizationTestFunction]



"""
Set of classes and functions to manipulate

ABINIT '.in' input files
ABINIT '.files' files
ABINIT '_OUT.nc' output files

"""
from . import task

from .abifiles import AbiFiles
from .input import AbinitInput, xyz2input
from .abinit import AbinitJob
from .output import AbinitOutput
from .run import AbinitRun
from .utils import psp_name
from .parser import parser
from .multibinit import Multibinit

# __all__ = filter(lambda s: not s.startswith('_'), dir())



"""
Module for support V_sim ascii fileformat
Contains routines to load .ascii files and
create pychemia Structure objects and to save
back to .ascii files

This code was originally created for ASE
"""

import re as _re
from pychemia.utils.constants import bohr_angstrom
from pychemia.core import Structure


def load(filep):
    """
    Read an V_sim .ascii file and returns a pychemia
    Structure object

    Args:
       filep: (string) Path to a .ascii file or an
             actual file-like object

    Returns:
       struct: (object) A pychemia Structure object
    """

    if isinstance(filep, str):
        f = open(filep)
    else:  # Assume it's a file-like object
        f = filep

    comment = f.readline()

    line = f.readline() + ' ' + f.readline()
    box = line.split()
    for i in range(len(box)):
        box[i] = float(box[i])

    keywords = []
    positions = []
    symbols = []

    re_comment = _re.compile('^\s*[#!]')
    re_node = _re.compile('^\s*\S+\s+\S+\s+\S+\s+\S+')

    while True:
        line = f.readline()

        if line == '':
            break  # EOF -> Exit

        p = re_comment.match(line)
        if p is not None:
            # remove comment character at the beginning of line
            line = line[p.end():].replace(',', ' ').lower()
            if line[:8] == "keyword:":
                keywords.extend(line[8:].split())

        elif re_node.match(line):
            unit = 1.0
            if not ("reduced" in keywords):
                if ("bohr" in keywords) or ("bohrd0" in keywords) or ("atomic" in keywords) or ("atomicd0" in keywords):
                    unit = bohr_angstrom

            fields = line.split()
            positions.append([unit * float(fields[0]),
                              unit * float(fields[1]),
                              unit * float(fields[2])])
            symbols.append(fields[3])

    f.close()

    if ("surface" in keywords) or ("freeBC" in keywords):
        raise NotImplementedError

    # create atoms object based on the information
    if "angdeg" in keywords:
        cell = cell.par_to_cell(box)
    else:
        unit = 1.0
        if ("bohr" in keywords) or ("bohrd0" in keywords) or ("atomic" in keywords) or ("atomicd0" in keywords):
            unit = bohr_angstrom
        cell = [[unit * box[0], 0.0, 0.0],
                [unit * box[1], unit * box[2], 0.0],
                [unit * box[3], unit * box[4], unit * box[5]]]

    if "reduced" in keywords:
        struct = Structure(cell=cell, reduced=positions, symbols=symbols, name=comment)
    else:
        struct = Structure(cell=cell, positions=positions, symbols=symbols, name=comment)

    return struct


def save(struct, filep, cartesian=True, long_format=True, angdeg=False):
    """
    Saves a pychemia Structure object in  V_sim .ascii fileformat
    in the simplest way, i.e. using all
    defaults with no optional keywords. In the first line we add the
    number of atoms, as this is used by certain code
    """

    if isinstance(filep, str):
        f = open(filep, 'w')
    else:  # Assume it's a 'file-like object'
        f = filep

    # write header (treated as a comment by v_sim
    f.write("%s\n" % struct.name)

    # write cell
    cell = struct.cell
    if angdeg:
        ddd = cell_to_cellpar(cell)
    else:
        ddd = cell_to_reduced(cell)

    f.write("%.14f %.14f %.14f\n" % (ddd[0], ddd[1], ddd[2]))
    f.write("%.14f %.14f %.14f\n" % (ddd[3], ddd[4], ddd[5]))

    if angdeg:
        f.write("#keyword: angdeg\n")

    # Write atom positions in scaled or cartesian coordinates
    if cartesian:
        coord = struct.positions
    else:
        f.write("#keyword: reduced\n")
        coord = struct.reduced

    if long_format:
        cform = ' %19.16f'
    else:
        cform = ' %9.6f'

    symbols = struct.symbols
    for iatom, atom in enumerate(coord):
        f.write(' ')
        for dcoord in atom:
            f.write(cform % dcoord)
        f.write('  ' + symbols[iatom] + '\n')


def cell_to_reduced(full):
    """
    Transforms the given matrix full into a reduced array used by
    V_Sim to store box definition.

    translated from src/coreTools/toolMatrix.c
    subroutine tool_matrix_reducePrimitiveVectors
    """

    from numpy import zeros
    from numpy.linalg import norm

    xcoord = full[0].copy()

    # Compute the Y vector of the new basis, orthogonal to xcoord an coplanar with xcoord and old y vector
    u = zeros(3)
    u[0] = full[0][1] * full[1][2] - full[0][2] * full[1][1]
    u[1] = full[0][2] * full[1][0] - full[0][0] * full[1][2]
    u[2] = full[0][0] * full[1][1] - full[0][1] * full[1][0]

    deltaij = xcoord[0] * u[1] - xcoord[1] * u[0]
    if deltaij != 0.0:
        i = 0
        j = 1
        k = 2
    else:
        deltaij = xcoord[0] * u[2] - xcoord[2] * u[0]
        if deltaij != 0.0:
            i = 0
            j = 2
            k = 1
        else:
            deltaij = xcoord[1] * u[2] - xcoord[2] * u[1]
            if deltaij != 0.0:
                i = 1
                j = 2
                k = 0
            else:
                # Error
                return None

    y = zeros(3)
    y[k] = -1.0
    y[i] = (xcoord[k] * u[j] - xcoord[j] * u[k]) / deltaij
    y[j] = (xcoord[i] * u[k] - xcoord[k] * u[i]) / deltaij

    # We need to turn Y if y.Y is negative
    fnorm = full[1][0] * y[0] + full[1][1] * y[1] + full[1][2] * y[2]
    if fnorm < 0.0:
        y *= -1.

    # Compute the new Z vector in order to form a direct orthogonal
    # basis with xcoord and Y
    z = zeros(3)
    z[0] = xcoord[1] * y[2] - xcoord[2] * y[1]
    z[1] = xcoord[2] * y[0] - xcoord[0] * y[2]
    z[2] = xcoord[0] * y[1] - xcoord[1] * y[0]

    # Normalize vectors
    xcoord /= norm(xcoord)
    y /= norm(y)
    z /= norm(z)

    # Compute the reduce value for the basis.
    reduced = zeros(6)
    reduced[0] = xcoord[0] * full[0][0] + xcoord[1] * full[0][1] + xcoord[2] * full[0][2]
    reduced[1] = xcoord[0] * full[1][0] + xcoord[1] * full[1][1] + xcoord[2] * full[1][2]
    reduced[2] = y[0] * full[1][0] + y[1] * full[1][1] + y[2] * full[1][2]
    reduced[3] = xcoord[0] * full[2][0] + xcoord[1] * full[2][1] + xcoord[2] * full[2][2]
    reduced[4] = y[0] * full[2][0] + y[1] * full[2][1] + y[2] * full[2][2]
    reduced[5] = z[0] * full[2][0] + z[1] * full[2][1] + z[2] * full[2][2]

    return reduced
